@@ -1,6 +1,39 @@
+/*  Part of SWI-Prolog
+
+    Author:        Jan Wielemaker
+    E-mail:        J.Wielemaker@vu.nl
+    WWW:           http://www.swi-prolog.org
+    Copyright (c)  2007-2017, VU University Amsterdam
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions
+    are met:
+
+    1. Redistributions of source code must retain the above copyright
+       notice, this list of conditions and the following disclaimer.
+
+    2. Redistributions in binary form must reproduce the above copyright
+       notice, this list of conditions and the following disclaimer in
+       the documentation and/or other materials provided with the
+       distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+    POSSIBILITY OF SUCH DAMAGE.
+*/
+
 :- module(libtotex,
-	  [ libtotex/3,
-	    libtotex/0
+	  [ libtotex/3
 	  ]).
 :- use_module(library(doc_latex)).
 :- use_module(library(pldoc/doc_process)).
@@ -9,6 +42,8 @@
 :- use_module(library(apply)).
 :- use_module(library(lists)).
 
+:- initialization(main, main).
+
 libtotex(Lib, Out, Options) :-
 	use_module(user:Lib),			% we want the operators in user
 	ensure_doc_loaded(Lib),
@@ -83,13 +118,6 @@ load_prolog([H|T0], [H|T]) :-
 	load_prolog(T0, T).
 
 
-%%	libtotex
-%
-%	Usage: swipl -q -s libtotex.pl -g libtotex -- file ...
-
-libtotex :-
-	main.
-
 main(Argv) :-
 	partition(is_option, Argv, OptArgs, Files),
 	maplist(to_option, OptArgs, AllOptions),
