@@ -3,35 +3,55 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2014, University of Amsterdam
-			      VU University Amsterdam
+    Copyright (c)  2001-2015, University of Amsterdam
+                              VU University Amsterdam
+    All rights reserved.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions
+    are met:
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+    1. Redistributions of source code must retain the above copyright
+       notice, this list of conditions and the following disclaimer.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    2. Redistributions in binary form must reproduce the above copyright
+       notice, this list of conditions and the following disclaimer in
+       the documentation and/or other materials provided with the
+       distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+    POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include <config.h>
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <errno.h>
+#ifdef O_PLMT
 #include <pthread.h>
+#endif
 #include "error.h"
 
+#ifdef O_PLMT
 #define LOCK(mf)   pthread_mutex_lock(&(mf)->mutex)
 #define UNLOCK(mf) pthread_mutex_unlock(&(mf)->mutex)
+#else
+#define LOCK(mf)
+#define UNLOCK(mf)
+#endif
 
 static atom_t ATOM_encoding;
 static atom_t ATOM_unknown;
@@ -51,6 +71,7 @@ static atom_t ATOM_insert;
 static atom_t ATOM_free_on_close;
 
 #define MEMFILE_MAGIC	0x5624a6b3L
+#define MEMFILE_CMAGIC	0x5624a6b7L
 #define NOSIZE ((size_t)-1)
 
 #define	V_CHARCOUNT	0x01
@@ -79,7 +100,9 @@ typedef struct
   atom_t	symbol;			/* <memory_file>(%p) */
   atom_t	atom;			/* Created from atom */
   atom_t	mode;			/* current open mode */
+#ifdef O_PLMT
   pthread_mutex_t mutex;		/* Our lock */
+#endif
   int		magic;			/* MEMFILE_MAGIC */
   int		free_on_close;		/* free if it is closed */
   IOENC		encoding;		/* encoding of the data */
@@ -166,6 +189,7 @@ get_memfile(term_t handle, memfile **mfp)
       return TRUE;
     }
 
+    UNLOCK(mf);
     PL_permission_error("access", "freed_memory_file", handle);
     return FALSE;
   }
@@ -205,7 +229,13 @@ new_memory_file(term_t handle)
 
   m->magic    = MEMFILE_MAGIC;
   m->encoding = ENC_UTF8;
+  m->data     = NULL;
+  m->atom     = 0;
+  m->symbol   = 0;
+  m->stream   = NULL;
+#ifdef O_PLMT
   pthread_mutex_init(&m->mutex, NULL);
+#endif
 
   if ( unify_memfile(handle, m) )
     return TRUE;
@@ -235,7 +265,10 @@ clean_memory_file(memfile *m)
 static int
 destroy_memory_file(memfile *m)
 { clean_memory_file(m);
+#ifdef O_PLMT
   pthread_mutex_destroy(&m->mutex);
+#endif
+  m->magic = MEMFILE_CMAGIC;
   free(m);
 
   return TRUE;
@@ -603,7 +636,7 @@ open_memory_file4(term_t handle, term_t mode, term_t stream, term_t options)
       term_t head = PL_new_term_ref();
 
       while(PL_get_list(tail, head, tail))
-      { int arity;
+      { size_t arity;
 	atom_t name;
 
 	if ( PL_get_name_arity(head, &name, &arity) && arity == 1 )
@@ -868,7 +901,9 @@ atom_to_memory_file(term_t atom, term_t handle)
       m->gap_start  = m->end;
     }
 
+#ifdef O_PLMT
     pthread_mutex_init(&m->mutex, NULL);
+#endif
 
     if ( unify_memfile(handle, m) )
     { return TRUE;
