@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2014-2015, VU University Amsterdam
+    Copyright (c)  2014-2017, VU University Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -133,8 +133,6 @@ static void
 free_ws_context(ws_context *ctx)
 { if ( ctx->stream->upstream )
     Sset_filter(ctx->stream, NULL);
-  else
-    PL_release_stream(ctx->stream);
 
   if ( ctx->data )
     free(ctx->data);
@@ -612,7 +610,7 @@ ws_open(term_t org, term_t new, term_t options)
 { term_t tail = PL_copy_term_ref(options);
   term_t head = PL_new_term_ref();
   ws_context *ctx;
-  IOSTREAM *s, *s2;
+  IOSTREAM *s = NULL, *s2 = NULL;
   ws_mode mode = WS_CLIENT;
   int close_parent = TRUE;
   int bufsize = 0;
@@ -660,7 +658,7 @@ ws_open(term_t org, term_t new, term_t options)
     return FALSE;			/* Error */
 
   if ( !(ctx = alloc_ws_context(s)) )
-    return FALSE;
+    goto error;
 
   PL_register_atom(subprotocol);
 
@@ -671,9 +669,7 @@ ws_open(term_t org, term_t new, term_t options)
   if ( !(s2 = Snew(ctx,
 		   (s->flags&WS_COPY_FLAGS)|SIO_FBUF,
 		   &ws_functions)) )
-  { free_ws_context(ctx);			/* no memory */
-    return FALSE;
-  }
+    goto error;
 
   ctx->ws_stream = s2;
   if ( bufsize > 0 )
@@ -686,10 +682,18 @@ ws_open(term_t org, term_t new, term_t options)
     PL_release_stream(s);
 
     return TRUE;
-  } else					/* stack overflow */
-  { Sclose(s2);
-    return FALSE;
   }
+
+error:
+  if ( s )
+    PL_release_stream(s);
+  if ( s2 )
+  { ctx->close_parent = FALSE;
+    Sclose(s2);
+  } else if ( ctx )
+    free_ws_context(ctx);
+
+  return FALSE;
 }
 
 
@@ -754,10 +758,11 @@ ws_header(char *hdr, ws_context *ctx, int fin, int mask, size_t payload_len)
     hdr_len += 2;
   } else
   { int i;
+    uint64_t plen64 = payload_len;
 
     hdr[1] = ( (masked<<7) | 127 );
     for(i=0; i<8; i++)
-      hdr[2+i] = (payload_len >> ((7-i)*8)) & 0xff;
+      hdr[2+i] = (plen64 >> ((7-i)*8)) & 0xff;
     hdr_len += 8;
   }
   if ( masked )
