@@ -3,7 +3,7 @@ package org.jpl7;
 
 class Version {
 	public final int major = 7;
-	public final int minor = 0;
-	public final int patch = 1; // 15/Jun/2015
+	public final int minor = 4; // jref as blob
+	public final int patch = 0;
 	public final String status = "alpha";
 }
