@@ -1,25 +1,36 @@
 /*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        J.Wielemaker@cs.vu.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2002-2012, University of Amsterdam
-			      VU University Amsterdam
+    Copyright (c)  2006-2015, University of Amsterdam
+                              VU University Amsterdam
+    All rights reserved.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions
+    are met:
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+    1. Redistributions of source code must retain the above copyright
+       notice, this list of conditions and the following disclaimer.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301  USA
+    2. Redistributions in binary form must reproduce the above copyright
+       notice, this list of conditions and the following disclaimer in
+       the documentation and/or other materials provided with the
+       distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+    POSSIBILITY OF SUCH DAMAGE.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -1007,7 +1018,7 @@ rdf_keys_in_literal_map_proteced(atom_map *map, term_t spec, term_t keys)
 { term_t tail = PL_copy_term_ref(keys);
   term_t head = PL_new_term_ref();
   atom_t name;
-  int arity;
+  size_t arity;
 
   if ( !PL_get_name_arity(spec, &name, &arity) )
     PL_type_error("key-specifier", spec);
@@ -1038,9 +1049,9 @@ rdf_keys_in_literal_map_proteced(atom_map *map, term_t spec, term_t keys)
     if ( (data = skiplist_find(&map->list, &search)) )
     { intptr_t size = (intptr_t)data->values.size;
 
-      assert(size > 0);
-
-      return PL_unify_integer(keys, size);
+      if ( size > 0 )
+	return PL_unify_integer(keys, size);
+      assert(size == 0);
     }
 
     return FALSE;
@@ -1050,7 +1061,7 @@ rdf_keys_in_literal_map_proteced(atom_map *map, term_t spec, term_t keys)
     skiplist_enum state;
     node_data *data;
     node_data_ex search;
-    int match = (name == ATOM_prefix ? STR_MATCH_PREFIX : STR_MATCH_EXACT);
+    int match = (name == ATOM_prefix ? STR_MATCH_PREFIX : STR_MATCH_ICASE);
 
     _PL_get_arg(1, spec, a);
     if ( !PL_get_atom_ex(a, &prefix) )
