@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2000-2016, University of Amsterdam
+    Copyright (c)  2000-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -500,19 +500,19 @@ pl_set_sgml_parser(term_t parser, term_t option)
       return sgml2pl_error(ERR_TYPE, "atom", a);
 
     if ( streq(s, "xml") )
-      set_dialect_dtd(p->dtd, DL_XML);
+      set_dialect_dtd(p->dtd, p, DL_XML);
     else if ( streq(s, "xmlns") )
-      set_dialect_dtd(p->dtd, DL_XMLNS);
+      set_dialect_dtd(p->dtd, p, DL_XMLNS);
     else if ( streq(s, "sgml") )
-      set_dialect_dtd(p->dtd, DL_SGML);
+      set_dialect_dtd(p->dtd, p, DL_SGML);
     else if ( streq(s, "html") || streq(s, "html4") )
-      set_dialect_dtd(p->dtd, DL_HTML);
+      set_dialect_dtd(p->dtd, p, DL_HTML);
     else if ( streq(s, "html5") )
-      set_dialect_dtd(p->dtd, DL_HTML5);
+      set_dialect_dtd(p->dtd, p, DL_HTML5);
     else if ( streq(s, "xhtml") )
-      set_dialect_dtd(p->dtd, DL_XHTML);
+      set_dialect_dtd(p->dtd, p, DL_XHTML);
     else if ( streq(s, "xhtml5") )
-      set_dialect_dtd(p->dtd, DL_XHTML5);
+      set_dialect_dtd(p->dtd, p, DL_XHTML5);
     else
       return sgml2pl_error(ERR_DOMAIN, "sgml_dialect", a);
   } else if ( PL_is_functor(option, FUNCTOR_space1) )
@@ -1803,11 +1803,11 @@ pl_open_dtd(term_t ref, term_t options, term_t stream)
 	return sgml2pl_error(ERR_TYPE, "atom", a);
 
       if ( streq(s, "xml") )
-	set_dialect_dtd(dtd, DL_XML);
+	set_dialect_dtd(dtd, p, DL_XML);
       else if ( streq(s, "xmlns") )
-	set_dialect_dtd(dtd, DL_XMLNS);
+	set_dialect_dtd(dtd, p, DL_XMLNS);
       else if ( streq(s, "sgml") )
-	set_dialect_dtd(dtd, DL_SGML);
+	set_dialect_dtd(dtd, p, DL_SGML);
       else
 	return sgml2pl_error(ERR_DOMAIN, "sgml_dialect", a);
     }
@@ -2039,7 +2039,7 @@ pl_sgml_parse(term_t parser, term_t options)
     { term_t a = PL_new_term_ref();
 
       _PL_get_arg(1, head, a);
-      if ( !PL_get_stream_handle(a, &in) )
+      if ( !PL_get_stream(a, &in, SIO_INPUT) )
 	return FALSE;
       release = in;
     }/* else ignored option */
@@ -2116,6 +2116,9 @@ pl_sgml_parse(term_t parser, term_t options)
 	    goto stopped;
 	  c = CR;
 	}
+      } else if ( Sferror(in) )
+      { rc = FALSE;
+	goto out;
       }
 
       if ( has_content_length && in->position )
@@ -2141,13 +2144,11 @@ pl_sgml_parse(term_t parser, term_t options)
 
   out:
     if ( release )
-      PL_release_stream(release);
+      rc = PL_release_stream(release) && rc;
 
     reset_url_cache();
-    if ( pd->tail )
-    { if ( !PL_unify_nil(pd->tail) )
-	return FALSE;
-    }
+    if ( pd->tail && rc )
+      rc = PL_unify_nil(pd->tail);
 
     if ( recursive )
     { p->closure = oldpd;
