@@ -38,6 +38,10 @@
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
 #include <openssl/hmac.h>
+#include <openssl/rand.h>
+#ifdef HAVE_OPENSSL_KDF_H
+#include <openssl/kdf.h>
+#endif
 #include "cryptolib.c"
 
 static atom_t ATOM_sslv23;
@@ -54,6 +58,11 @@ static atom_t ATOM_sha384;
 static atom_t ATOM_sha512;
 static atom_t ATOM_blake2s256;
 static atom_t ATOM_blake2b512;
+static atom_t ATOM_sha3_224;
+static atom_t ATOM_sha3_256;
+static atom_t ATOM_sha3_384;
+static atom_t ATOM_sha3_512;
+static atom_t ATOM_ripemd160;
 
 static atom_t ATOM_pkcs1;
 static atom_t ATOM_pkcs1_oaep;
@@ -73,14 +82,44 @@ typedef enum
 } crypt_mode_t;
 
 
+                 /***************************
+                 *       RANDOM BYTES       *
+                 ****************************/
+
+static foreign_t
+pl_crypto_n_random_bytes(term_t tn, term_t tout)
+{ size_t len;
+  int rc;
+  unsigned char *buffer;
+
+  if ( !PL_get_size_ex(tn, &len) )
+    return FALSE;
+
+  if ( !(buffer = malloc(len)) )
+    return PL_resource_error("memory");
+
+  if ( RAND_bytes(buffer, len) == 0 )
+  { free(buffer);
+    return raise_ssl_error(ERR_get_error());
+  }
+
+  rc = PL_unify_chars(tout, PL_CODE_LIST|REP_ISO_LATIN_1,
+		      len, (const char *) buffer);
+
+  free(buffer);
+
+  return rc;
+}
+
+
                  /***************************
                  *         HASHING          *
                  ****************************/
 
 
-#define CONTEXT_MAGIC (~ 0x53481284L)
+#define HASH_CONTEXT_MAGIC (~ 0x53481284L)
 
-typedef struct context
+typedef struct hash_context
 { int             magic;
   atom_t          atom;
   IOENC           encoding;
@@ -94,35 +133,34 @@ typedef struct context
   EVP_MD_CTX     *ctx;
   HMAC_CTX       *hmac_ctx;
   char           *hmac_key;
-} PL_CRYPTO_CONTEXT;
+} PL_CRYPTO_HASH_CONTEXT;
 
-static int
-free_crypto_context(PL_CRYPTO_CONTEXT *c)
-{
-  EVP_MD_CTX_free(c->ctx);
-  free(c);
+static void
+free_crypto_hash_context(PL_CRYPTO_HASH_CONTEXT *c)
+{ EVP_MD_CTX_free(c->ctx);
   free(c->hmac_key);
 #ifdef HAVE_HMAC_CTX_FREE
   HMAC_CTX_free(c->hmac_ctx);
 #endif
+  free(c);
+}
+
+static int
+release_hash_context(atom_t atom)
+{ PL_CRYPTO_HASH_CONTEXT **cp = PL_blob_data(atom, NULL, NULL);
+  PL_CRYPTO_HASH_CONTEXT  *c = *cp;
+
+  ssl_deb(4, "Releasing PL_CRYPTO_HASH_CONTEXT %p\n", c);
+  free_crypto_hash_context(c);
   return TRUE;
 }
 
 static int
-release_context(atom_t atom)
-{ PL_CRYPTO_CONTEXT *c;
-  size_t size;
-
-  c = PL_blob_data(atom, &size, NULL);
-  ssl_deb(4, "Releasing PL_CRYPTO_CONTEXT %p\n", c);
-  free_crypto_context(c);
-  return TRUE;
-}
-
-static int
-compare_context(atom_t a, atom_t b)
-{ PL_CRYPTO_CONTEXT* *c1 = PL_blob_data(a, NULL, NULL);
-  PL_CRYPTO_CONTEXT* *c2 = PL_blob_data(b, NULL, NULL);
+compare_hash_context(atom_t a, atom_t b)
+{ PL_CRYPTO_HASH_CONTEXT **cp1 = PL_blob_data(a, NULL, NULL);
+  PL_CRYPTO_HASH_CONTEXT **cp2 = PL_blob_data(b, NULL, NULL);
+  PL_CRYPTO_HASH_CONTEXT  *c1 = *cp1;
+  PL_CRYPTO_HASH_CONTEXT  *c2 = *cp2;
 
   return ( c1 > c2 ?  1 :
            c1 < c2 ? -1 : 0
@@ -130,68 +168,117 @@ compare_context(atom_t a, atom_t b)
 }
 
 static int
-write_context(IOSTREAM *s, atom_t symbol, int flags)
-{ PL_CRYPTO_CONTEXT *c = PL_blob_data(symbol, NULL, NULL);
+write_hash_context(IOSTREAM *s, atom_t symbol, int flags)
+{ PL_CRYPTO_HASH_CONTEXT **cp = PL_blob_data(symbol, NULL, NULL);
+  PL_CRYPTO_HASH_CONTEXT  *c  = *cp;
 
-  Sfprintf(s, "<crypto_context>(%p)", c);
+  Sfprintf(s, "<crypto_hash_context>(%p)", c);
 
   return TRUE;
 }
 
 static void
-acquire_context(atom_t atom)
-{ ssl_deb(4, "Acquire on atom %d\n", atom);
+acquire_hash_context(atom_t atom)
+{ PL_CRYPTO_HASH_CONTEXT **cp = PL_blob_data(atom, NULL, NULL);
+  PL_CRYPTO_HASH_CONTEXT  *c  = *cp;
+
+  c->atom = atom;
 }
 
-static PL_blob_t crypto_context_type =
+static PL_blob_t crypto_hash_context_type =
 { PL_BLOB_MAGIC,
-  PL_BLOB_NOCOPY,
-  "crypto_context",
-  release_context,
-  compare_context,
-  write_context,
-  acquire_context
+  0,
+  "crypto_hash_context",
+  release_hash_context,
+  compare_hash_context,
+  write_hash_context,
+  acquire_hash_context
 };
 
+
 static int
-put_context(term_t tcontext, PL_CRYPTO_CONTEXT *context)
-{ return PL_unify_atom(tcontext, context->atom);
+unify_hash_context(term_t tcontext, PL_CRYPTO_HASH_CONTEXT *context)
+{ if ( PL_unify_blob(tcontext, &context, sizeof(context), &crypto_hash_context_type) )
+    return TRUE;
+
+  free_crypto_hash_context(context);
+  if ( !PL_exception(0) )
+    return PL_uninstantiation_error(tcontext);
+
+  return FALSE;
 }
 
-static int
-register_context(term_t tcontext, PL_CRYPTO_CONTEXT *context)
-{ term_t blob = PL_new_term_ref();
-  int rc;
-
-  PL_put_blob(blob, context, sizeof(void*), &crypto_context_type);
-  rc = PL_get_atom(blob, &context->atom);
-  assert(rc);
-  ssl_deb(4, "Atom created: %d\n", context->atom);
-  return put_context(tcontext, context);
-}
 
 static int
-get_context(term_t tcontext, PL_CRYPTO_CONTEXT **context)
+get_hash_context(term_t tcontext, PL_CRYPTO_HASH_CONTEXT **context)
 { PL_blob_t *type;
   void *data;
 
   if ( PL_get_blob(tcontext, &data, NULL, &type) &&
-       type == &crypto_context_type )
-  { PL_CRYPTO_CONTEXT *c = data;
+       type == &crypto_hash_context_type )
+  { PL_CRYPTO_HASH_CONTEXT *c = *(PL_CRYPTO_HASH_CONTEXT**)data;
 
-    assert(c->magic == CONTEXT_MAGIC);
+    assert(c->magic == HASH_CONTEXT_MAGIC);
     *context = c;
 
     return TRUE;
   }
 
-  return PL_type_error("crypto_context", tcontext);
+  return PL_type_error("crypto_hash_context", tcontext);
+}
+
+typedef struct algorithm_pair {
+  atom_t a_algorithm;
+  const EVP_MD *algorithm;
+} ALGORITHM_PAIR;
+
+#define ALGO(a) { ATOM_## a , EVP_## a() }
+#define NELEMS(array) (sizeof(array)/sizeof((array)[0]))
+
+static int
+get_hash_algorithm(atom_t a_algorithm, const EVP_MD **algorithm)
+{ int i;
+  ALGORITHM_PAIR algorithms[] =
+    { ALGO(md5), ALGO(ripemd160),
+#if defined(HAVE_EVP_BLAKE2B512) && defined(HAVE_EVP_BLAKE2S256)
+      ALGO(blake2s256), ALGO(blake2b512),
+#endif
+#if defined(HAVE_EVP_SHA3_224) && defined(HAVE_EVP_SHA3_256) && \
+    defined(HAVE_EVP_SHA3_384) && defined(HAVE_EVP_SHA3_512)
+      ALGO(sha3_224), ALGO(sha3_256), ALGO(sha3_384), ALGO(sha3_512),
+#endif
+      ALGO(sha1), ALGO(sha224), ALGO(sha256), ALGO(sha384), ALGO(sha512)
+    };
+
+  for (i = 0; i < NELEMS(algorithms); i++)
+  { if (a_algorithm == algorithms[i].a_algorithm)
+    { *algorithm = algorithms[i].algorithm;
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static int
+get_text_representation(term_t t, int *rep)
+{ atom_t a;
+
+  if ( PL_get_atom_ex(t, &a) )
+  { if      ( a == ATOM_octet ) *rep = REP_ISO_LATIN_1;
+    else if ( a == ATOM_utf8  ) *rep = REP_UTF8;
+    else if ( a == ATOM_text  ) *rep = REP_MB;
+    else return PL_domain_error("encoding", t);
+
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
 
-
 static int
-hash_options(term_t options, PL_CRYPTO_CONTEXT *result)
+hash_options(term_t options, PL_CRYPTO_HASH_CONTEXT *result)
 { term_t opts = PL_copy_term_ref(options);
   term_t opt = PL_new_term_ref();
 
@@ -213,48 +300,26 @@ hash_options(term_t options, PL_CRYPTO_CONTEXT *result)
 
         if ( !PL_get_atom_ex(a, &a_algorithm) )
           return FALSE;
-        if ( a_algorithm == ATOM_sha1  )
-        { result->algorithm = EVP_sha1();
-        } else if ( a_algorithm == ATOM_sha224 )
-        { result->algorithm = EVP_sha224();
-        } else if ( a_algorithm == ATOM_sha256 )
-        { result->algorithm = EVP_sha256();
-        } else if ( a_algorithm == ATOM_sha384 )
-        { result->algorithm = EVP_sha384();
-        } else if ( a_algorithm == ATOM_sha512 )
-        { result->algorithm = EVP_sha512();
-        } else if ( a_algorithm == ATOM_md5 )
-        { result->algorithm = EVP_md5();
-#if defined(HAVE_EVP_BLAKE2B512) && defined(HAVE_EVP_BLAKE2S256)
-        } else if ( a_algorithm == ATOM_blake2b512 )
-        {  result->algorithm = EVP_blake2b512();
-        } else if ( a_algorithm == ATOM_blake2s256 )
-        { result->algorithm = EVP_blake2s256();
-#endif
-        } else
+
+        if ( !get_hash_algorithm(a_algorithm, &result->algorithm) )
           return PL_domain_error("algorithm", a);
       } else if ( aname == ATOM_hmac )
       { size_t key_len;
         char *key;
 
         if ( !PL_get_nchars(a, &key_len, &key,
-                      CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION) )
           return FALSE;
         result->hmac_key = ssl_strdup(key);
       } else if ( aname == ATOM_close_parent )
       { if ( !PL_get_bool_ex(a, &result->close_parent) )
           return FALSE;
       } else if ( aname == ATOM_encoding )
-      { atom_t a_enc;
+      {  int rep;
+         if ( !get_text_representation(a, &rep) )
+           return PL_domain_error("encoding", a);
 
-        if ( !PL_get_atom_ex(a, &a_enc) )
-          return FALSE;
-        if ( a_enc == ATOM_utf8 )
-          result->encoding = REP_UTF8;
-        else if ( a_enc == ATOM_octet )
-          result->encoding = REP_ISO_LATIN_1;
-        else
-          return PL_domain_error("encoding", a);
+         result->encoding = ( rep == REP_UTF8 ) ? REP_UTF8 : REP_ISO_LATIN_1;
       }
     } else
     { return PL_type_error("option", opt);
@@ -269,16 +334,16 @@ hash_options(term_t options, PL_CRYPTO_CONTEXT *result)
 
 
 static foreign_t
-pl_crypto_context_new(term_t tcontext, term_t options)
+pl_crypto_hash_context_new(term_t tcontext, term_t options)
 {
-  PL_CRYPTO_CONTEXT *context = NULL;
+  PL_CRYPTO_HASH_CONTEXT *context = NULL;
 
   context = malloc(sizeof(*context));
 
   if ( !context )
     return FALSE;
 
-  context->magic    = CONTEXT_MAGIC;
+  context->magic    = HASH_CONTEXT_MAGIC;
   context->ctx      = NULL;
   context->hmac_ctx = NULL;
   context->hmac_key = NULL;
@@ -309,16 +374,16 @@ pl_crypto_context_new(term_t tcontext, term_t options)
     }
   }
 
-  return register_context(tcontext, context);
+  return unify_hash_context(tcontext, context);
 }
 
 static foreign_t
-pl_crypto_context_copy(term_t tin, term_t tout)
+pl_crypto_hash_context_copy(term_t tin, term_t tout)
 {
-  PL_CRYPTO_CONTEXT *in, *out;
+  PL_CRYPTO_HASH_CONTEXT *in, *out;
   int rc = 0;
 
-  if ( !get_context(tin, &in) )
+  if ( !get_hash_context(tin, &in) )
     return FALSE;
 
   out = malloc(sizeof(*out));
@@ -326,7 +391,7 @@ pl_crypto_context_copy(term_t tin, term_t tout)
   if ( !out )
     return FALSE;
 
-  out->magic = CONTEXT_MAGIC;
+  out->magic = HASH_CONTEXT_MAGIC;
   out->hmac_key = ssl_strdup(in->hmac_key);
 
   out->encoding = in->encoding;
@@ -357,12 +422,12 @@ pl_crypto_context_copy(term_t tin, term_t tout)
   out->hmac_ctx = NULL;
 #endif
 
-  return register_context(tout, out) && rc;
+  return unify_hash_context(tout, out) && rc;
 }
 
 
 static int
-hash_append(PL_CRYPTO_CONTEXT *context, void *data, size_t size)
+hash_append(PL_CRYPTO_HASH_CONTEXT *context, void *data, size_t size)
 {
   if ( context->hmac_ctx )
     return HMAC_Update(context->hmac_ctx, data, size);
@@ -372,13 +437,13 @@ hash_append(PL_CRYPTO_CONTEXT *context, void *data, size_t size)
 
 
 static foreign_t
-pl_crypto_update_context(term_t from, term_t tcontext)
+pl_crypto_update_hash_context(term_t from, term_t tcontext)
 {
-  PL_CRYPTO_CONTEXT *context = NULL;
+  PL_CRYPTO_HASH_CONTEXT *context = NULL;
   size_t datalen;
   char *data;
 
-  if ( !get_context(tcontext, &context) )
+  if ( !get_hash_context(tcontext, &context) )
     return FALSE;
 
   if ( !PL_get_nchars(from, &datalen, &data,
@@ -390,13 +455,13 @@ pl_crypto_update_context(term_t from, term_t tcontext)
 }
 
 static foreign_t
-pl_crypto_context_hash(term_t tcontext, term_t hash)
+pl_crypto_hash_context_hash(term_t tcontext, term_t hash)
 {
-  PL_CRYPTO_CONTEXT *context = NULL;
+  PL_CRYPTO_HASH_CONTEXT *context = NULL;
   unsigned char digest[EVP_MAX_MD_SIZE];
   unsigned int len;
 
-  if ( !get_context(tcontext, &context) )
+  if ( !get_hash_context(tcontext, &context) )
     return FALSE;
 
   if ( context->hmac_ctx )
@@ -415,7 +480,7 @@ pl_crypto_context_hash(term_t tcontext, term_t hash)
 
 static ssize_t                          /* range-limited read */
 hash_read(void *handle, char *buf, size_t size)
-{ PL_CRYPTO_CONTEXT *ctx = handle;
+{ PL_CRYPTO_HASH_CONTEXT *ctx = handle;
   ssize_t rd;
 
   if ( (rd = Sfread(buf, sizeof(char), size, ctx->parent_stream)) >= 0 )
@@ -430,7 +495,7 @@ hash_read(void *handle, char *buf, size_t size)
 
 static ssize_t
 hash_write(void *handle, char *buf, size_t size)
-{ PL_CRYPTO_CONTEXT *ctx = handle;
+{ PL_CRYPTO_HASH_CONTEXT *ctx = handle;
   size_t written = 0;
 
   hash_append(ctx, buf, size);
@@ -450,7 +515,7 @@ hash_write(void *handle, char *buf, size_t size)
 
 static int
 hash_control(void *handle, int op, void *data)
-{ PL_CRYPTO_CONTEXT *ctx = handle;
+{ PL_CRYPTO_HASH_CONTEXT *ctx = handle;
 
   switch(op)
   { case SIO_SETENCODING:
@@ -466,7 +531,7 @@ hash_control(void *handle, int op, void *data)
 static int
 hash_close(void *handle)
 { int rc = 0;
-  PL_CRYPTO_CONTEXT *ctx = handle;
+  PL_CRYPTO_HASH_CONTEXT *ctx = handle;
 
   ctx->parent_stream->encoding = ctx->parent_encoding;
   if ( ctx->parent_stream->upstream )
@@ -475,7 +540,7 @@ hash_close(void *handle)
   if ( ctx->close_parent )
     rc = Sclose(ctx->parent_stream);
 
-  free_crypto_context(ctx);
+  free_crypto_hash_context(ctx);
 
   return rc;
 }
@@ -496,10 +561,10 @@ static IOFUNCTIONS hash_functions =
 
 static foreign_t
 pl_crypto_open_hash_stream(term_t org, term_t new, term_t tcontext)
-{ PL_CRYPTO_CONTEXT *context;
+{ PL_CRYPTO_HASH_CONTEXT *context;
   IOSTREAM *s, *s2;
 
-  if ( !get_context(tcontext, &context) )
+  if ( !get_hash_context(tcontext, &context) )
     return FALSE;
 
   if ( !PL_get_stream_handle(org, &s) )
@@ -536,13 +601,13 @@ pl_crypto_open_hash_stream(term_t org, term_t new, term_t tcontext)
 
 
 static foreign_t
-pl_crypto_stream_context(term_t stream, term_t tcontext)
+pl_crypto_stream_hash_context(term_t stream, term_t tcontext)
 { IOSTREAM *s;
   int rc;
 
   if ( PL_get_stream_handle(stream, &s) )
-  { PL_CRYPTO_CONTEXT *ctx = s->handle;
-    rc = register_context(tcontext, ctx);
+  { PL_CRYPTO_HASH_CONTEXT *ctx = s->handle;
+    rc = unify_hash_context(tcontext, ctx);
     PL_release_stream(s);
     return rc;
   }
@@ -550,6 +615,85 @@ pl_crypto_stream_context(term_t stream, term_t tcontext)
   return FALSE;
 }
 
+                 /***************************
+                 *    Hashes of passwords   *
+                 ****************************/
+
+static foreign_t
+pl_crypto_password_hash(term_t tpw, term_t tsalt, term_t titer, term_t tdigest)
+{ char *pw, *salt;
+  size_t pwlen, saltlen;
+  int iter;
+  const int DIGEST_LEN = 64;
+  unsigned char digest[DIGEST_LEN];
+
+  if ( !PL_get_nchars(tpw, &pwlen, &pw,
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION|REP_UTF8) ||
+       !PL_get_nchars(tsalt, &saltlen, &salt, CVT_LIST) ||
+       !PL_get_integer_ex(titer, &iter) )
+    return FALSE;
+
+  PKCS5_PBKDF2_HMAC((const char *) pw, pwlen,
+                    (const unsigned char *) salt, saltlen,
+                    iter, EVP_sha512(), DIGEST_LEN, digest);
+
+  return PL_unify_list_ncodes(tdigest, DIGEST_LEN, (char *) digest);
+}
+
+static foreign_t
+pl_crypto_data_hkdf(term_t tkey, term_t tsalt, term_t tinfo, term_t talg,
+                    term_t tencoding, term_t toutlen, term_t tout)
+{
+#if defined(HAVE_OPENSSL_KDF_H) && defined(EVP_PKEY_HKDF)
+  EVP_PKEY_CTX *pctx;
+  char *salt, *key, *info;
+  size_t keylen, infolen, outlen, saltlen;
+  int rep;
+  const EVP_MD *alg;
+  unsigned char *out;
+  atom_t a_algorithm;
+
+  if ( !PL_get_nchars(tsalt, &saltlen, &salt, CVT_LIST) ||
+       !PL_get_size_ex(toutlen, &outlen) ||
+       !PL_get_atom_ex(talg, &a_algorithm) )
+    return FALSE;
+
+  if ( !get_text_representation(tencoding, &rep) )
+    return PL_domain_error("encoding", tencoding);
+
+  if ( !PL_get_nchars(tkey, &keylen, &key,
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION|rep) ||
+       !PL_get_nchars(tinfo, &infolen, &info,
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION) )
+    return FALSE;
+
+  if ( !get_hash_algorithm(a_algorithm, &alg) )
+    return PL_domain_error("algorithm", a_algorithm);
+
+  if ( !(out = malloc(outlen)) )
+    return PL_resource_error("memory");
+
+  pctx = EVP_PKEY_CTX_new_id(EVP_PKEY_HKDF, NULL);
+
+  if ( (EVP_PKEY_derive_init(pctx) > 0) &&
+       (EVP_PKEY_CTX_set_hkdf_md(pctx, alg) > 0) &&
+       (EVP_PKEY_CTX_set1_hkdf_salt(pctx, salt, saltlen) > 0) &&
+       (EVP_PKEY_CTX_set1_hkdf_key(pctx, key, keylen) > 0) &&
+       (EVP_PKEY_CTX_add1_hkdf_info(pctx, info, infolen) > 0) &&
+       (EVP_PKEY_derive(pctx, out, &outlen) > 0) )
+  { int rc = PL_unify_list_ncodes(tout, outlen, (char *) out);
+    free(out);
+    EVP_PKEY_CTX_free(pctx);
+    return rc;
+  }
+
+  free(out);
+  EVP_PKEY_CTX_free(pctx);
+  return raise_ssl_error(ERR_get_error());
+#else
+  return ssl_missing("HKDF");
+#endif
+}
 
                  /***************************
                  *       Bignums & Keys     *
@@ -575,6 +719,7 @@ get_bn_arg(int a, term_t t, BIGNUM **bn)
   return FALSE;
 }
 
+#ifndef OPENSSL_NO_EC
 static int
 recover_ec(term_t t, EC_KEY **rec)
 {
@@ -612,12 +757,13 @@ recover_ec(term_t t, EC_KEY **rec)
   EC_KEY_free(ec);
   return FALSE;
 }
+#endif
 
 static int
 recover_rsa(term_t t, RSA** rsap)
 { RSA *rsa = RSA_new();
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
   if ( get_bn_arg(1, t, &rsa->n) &&
        get_bn_arg(2, t, &rsa->e) &&
        get_bn_arg(3, t, &rsa->d) &&
@@ -694,22 +840,6 @@ recover_public_key(term_t t, RSA** rsap)
                  *       OPTION PROCESSING      *
                  *******************************/
 
-static int
-get_text_representation(term_t t, int *rep)
-{ atom_t a;
-
-  if ( PL_get_atom_ex(t, &a) )
-  { if      ( a == ATOM_octet ) *rep = REP_ISO_LATIN_1;
-    else if ( a == ATOM_utf8  ) *rep = REP_UTF8;
-    else if ( a == ATOM_text  ) *rep = REP_MB;
-    else return PL_domain_error("encoding", t);
-
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 static int
 get_padding(term_t t, crypt_mode_t mode, int *padding)
 { atom_t a;
@@ -788,13 +918,14 @@ parse_options(term_t options_t, crypt_mode_t mode, int* rep, int* padding)
 
 static foreign_t
 pl_ecdsa_sign(term_t Private, term_t Data, term_t Enc, term_t Signature)
-{ unsigned char *data;
+{
+#ifndef OPENSSL_NO_ECDSA
+  unsigned char *data;
   size_t data_len;
   EC_KEY *key;
   ECDSA_SIG *sig;
   unsigned char *signature = NULL;
-  unsigned int signature_len;
-  int rc;
+  int signature_len, rc;
 
   if ( !recover_ec(Private, &key) ||
        !get_enc_text(Data, Enc, &data_len, &data) )
@@ -810,11 +941,16 @@ pl_ecdsa_sign(term_t Private, term_t Data, term_t Enc, term_t Signature)
   OPENSSL_free(signature);
 
   return rc;
+#else
+  return ssl_missing("ECDSA");
+#endif
 }
 
 static foreign_t
 pl_ecdsa_verify(term_t Public, term_t Data, term_t Enc, term_t Signature)
-{ unsigned char *data;
+{
+#ifndef OPENSSL_NO_ECDSA
+  unsigned char *data;
   size_t data_len;
   EC_KEY *key;
   ECDSA_SIG *sig;
@@ -842,6 +978,9 @@ pl_ecdsa_verify(term_t Public, term_t Data, term_t Enc, term_t Signature)
     return rc;
 
   return raise_ssl_error(ERR_get_error());
+#else
+  return ssl_missing("ECDSA");
+#endif
 }
 
 
@@ -1121,9 +1260,11 @@ pl_rsa_verify(term_t Public, term_t Type, term_t Enc,
 #endif
 
 static foreign_t
-pl_evp_decrypt(term_t ciphertext_t, term_t algorithm_t,
-	       term_t key_t, term_t iv_t, term_t plaintext_t,
-	       term_t options_t)
+pl_crypto_data_decrypt(term_t ciphertext_t, term_t algorithm_t,
+                       term_t key_t, term_t iv_t,
+                       term_t authtag_t,
+                       term_t plaintext_t,
+                       term_t options_t)
 { EVP_CIPHER_CTX* ctx = NULL;
   const EVP_CIPHER *cipher;
   char* key;
@@ -1136,8 +1277,12 @@ pl_evp_decrypt(term_t ciphertext_t, term_t algorithm_t,
   int cvt_flags = CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION;
   int rep = REP_UTF8;
   int padding = 1;
+#ifdef EVP_CTRL_AEAD_SET_TAG
+  char *authtag;
+  size_t authlen;
+#endif
 
-  if ( !parse_options(options_t, EVP_MODE, &rep, &padding))
+  if ( !parse_options(options_t, EVP_MODE, &rep, &padding) )
     return FALSE;
 
   if ( !PL_get_chars(key_t, &key, cvt_flags) ||
@@ -1154,6 +1299,17 @@ pl_evp_decrypt(term_t ciphertext_t, term_t algorithm_t,
   EVP_CIPHER_CTX_reset(ctx);
   EVP_DecryptInit_ex(ctx, cipher, NULL,
 		     (const unsigned char*)key, (const unsigned char*)iv);
+
+#ifdef EVP_CTRL_AEAD_SET_TAG
+  if ( PL_get_nchars(authtag_t, &authlen, &authtag, CVT_LIST) &&
+       ( authlen > 0 ) )
+  { if ( !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, authlen, authtag) )
+    { EVP_CIPHER_CTX_free(ctx);
+      return raise_ssl_error(ERR_get_error());
+    }
+  }
+#endif
+
   EVP_CIPHER_CTX_set_padding(ctx, padding);
   plaintext = PL_malloc(cipher_length + EVP_CIPHER_block_size(cipher));
   if ( EVP_DecryptUpdate(ctx, (unsigned char*)plaintext, &plain_length,
@@ -1162,7 +1318,12 @@ pl_evp_decrypt(term_t ciphertext_t, term_t algorithm_t,
     int rc;
     rc = EVP_DecryptFinal_ex(ctx, (unsigned char*)(plaintext + plain_length),
                               &last_chunk);
+
     EVP_CIPHER_CTX_free(ctx);
+
+    if ( !rc )
+      return raise_ssl_error(ERR_get_error());
+
     ERR_print_errors_fp(stderr);
     rc &= PL_unify_chars(plaintext_t, rep | PL_STRING, plain_length + last_chunk,
                          plaintext);
@@ -1173,13 +1334,15 @@ pl_evp_decrypt(term_t ciphertext_t, term_t algorithm_t,
   PL_free(plaintext);
   EVP_CIPHER_CTX_free(ctx);
 
-  return FALSE;
+  return raise_ssl_error(ERR_get_error());
 }
 
 static foreign_t
-pl_evp_encrypt(term_t plaintext_t, term_t algorithm_t,
-               term_t key_t, term_t iv_t, term_t ciphertext_t,
-	       term_t options_t)
+pl_crypto_data_encrypt(term_t plaintext_t, term_t algorithm_t,
+                       term_t key_t, term_t iv_t,
+                       term_t authlen_t, term_t authtag_t,
+                       term_t ciphertext_t,
+                       term_t options_t)
 { EVP_CIPHER_CTX* ctx = NULL;
   const EVP_CIPHER *cipher;
   char* key;
@@ -1191,8 +1354,16 @@ pl_evp_encrypt(term_t plaintext_t, term_t algorithm_t,
   size_t plain_length;
   int cvt_flags = CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION;
   int rep = REP_UTF8;
+  int padding = 1;
+  int authlen;
+  const int MAX_AUTHLEN = 256;
+#ifdef EVP_CTRL_AEAD_SET_TAG
+  char authtag[MAX_AUTHLEN];
+#endif
 
-  if ( !parse_options(options_t, EVP_MODE, &rep, NULL))
+  if ( !parse_options(options_t, EVP_MODE, &rep, &padding) ||
+       !PL_get_integer_ex(authlen_t, &authlen) ||
+       ( authlen > MAX_AUTHLEN ) )
     return FALSE;
 
   if ( !PL_get_chars(key_t, &key, cvt_flags) ||
@@ -1209,15 +1380,27 @@ pl_evp_encrypt(term_t plaintext_t, term_t algorithm_t,
   EVP_CIPHER_CTX_reset(ctx);
   EVP_EncryptInit_ex(ctx, cipher, NULL,
 		     (const unsigned char*)key, (const unsigned char*)iv);
-
+  EVP_CIPHER_CTX_set_padding(ctx, padding);
   ciphertext = PL_malloc(plain_length + EVP_CIPHER_block_size(cipher));
   if ( EVP_EncryptUpdate(ctx, (unsigned char*)ciphertext, &cipher_length,
                          (unsigned char*)plaintext, plain_length) == 1 )
   { int last_chunk;
     int rc;
 
-    EVP_EncryptFinal_ex(ctx, (unsigned char*)(ciphertext + cipher_length),
-			&last_chunk);
+    if ( !EVP_EncryptFinal_ex(ctx, (unsigned char*)(ciphertext + cipher_length),
+                              &last_chunk) )
+      return raise_ssl_error(ERR_get_error());
+
+#ifdef EVP_CTRL_AEAD_SET_TAG
+    if ( authlen >= 0 )
+    { if ( !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_GET_TAG, authlen, authtag) )
+        return raise_ssl_error(ERR_get_error());
+
+      if ( !PL_unify_list_ncodes(authtag_t, authlen, authtag) )
+        return FALSE;
+    }
+#endif
+
     EVP_CIPHER_CTX_free(ctx);
     rc = PL_unify_chars(ciphertext_t,  PL_STRING|REP_ISO_LATIN_1,
 			cipher_length + last_chunk, ciphertext);
@@ -1228,9 +1411,368 @@ pl_evp_encrypt(term_t plaintext_t, term_t algorithm_t,
   PL_free(ciphertext);
   EVP_CIPHER_CTX_free(ctx);
 
+  return raise_ssl_error(ERR_get_error());
+}
+
+
+          /*******************************************
+          *      MODULAR MULTIPLICATIVE INVERSE      *
+          ********************************************/
+
+static foreign_t
+pl_crypto_modular_inverse(term_t tx, term_t tm, term_t tout)
+{ BIGNUM *x = NULL, *m = NULL, *r = NULL;
+  BN_CTX *ctx = NULL;
+  char *hex = NULL;
+  int rc, ssl_err = FALSE;
+
+  if ( get_bn_arg(1, tx, &x) &&
+       get_bn_arg(1, tm, &m)  &&
+       ( ctx = BN_CTX_new() ) &&
+       ( r = BN_mod_inverse(NULL, x, m, ctx) ) &&
+       ( hex = BN_bn2hex(r) ) )
+  { rc = PL_unify_chars(tout, PL_STRING|REP_ISO_LATIN_1, strlen(hex), hex);
+  } else
+  { ssl_err = TRUE;
+  }
+
+  OPENSSL_free(hex);
+  BN_free(x);
+  BN_free(m);
+  BN_free(r);
+  BN_CTX_free(ctx);
+
+  if ( ssl_err )
+  { return raise_ssl_error(ERR_get_error());
+  }
+
+  return rc;
+}
+
+                 /*******************
+                 *      PRIMES      *
+                 ********************/
+
+static foreign_t
+pl_crypto_generate_prime(term_t tbits, term_t tprime, term_t tsafe,
+                         term_t toptions)
+{ BIGNUM *prime = NULL;
+  int bits, safe;
+  char *hex = NULL;
+  int rc, ssl_err = FALSE;
+
+  if ( !PL_get_integer_ex(tbits, &bits) )
+    return FALSE;
+
+  if ( !PL_get_bool_ex(tsafe, &safe) )
+    return FALSE;
+
+  if ( ( prime = BN_new() ) &&
+       ( BN_generate_prime_ex(prime, bits, safe, NULL, NULL, NULL ) ) &&
+       ( hex = BN_bn2hex(prime) ) )
+  { rc = PL_unify_chars(tprime, PL_STRING|REP_ISO_LATIN_1, strlen(hex), hex);
+  } else
+  { ssl_err = TRUE;
+  }
+
+  OPENSSL_free(hex);
+  BN_free(prime);
+
+  if ( ssl_err )
+  { return raise_ssl_error(ERR_get_error());
+  }
+
+  return rc;
+}
+
+static foreign_t
+pl_crypto_is_prime(term_t tprime, term_t tnchecks)
+{ BIGNUM *prime = NULL;
+  BN_CTX *ctx = NULL;
+  int nchecks;
+  int ret = -1;
+
+  if ( !PL_get_integer_ex(tnchecks, &nchecks) )
+    return FALSE;
+
+  nchecks = ( nchecks < 0 ) ? BN_prime_checks : nchecks;
+
+  if ( ( ctx = BN_CTX_new() ) &&
+       get_bn_arg(1, tprime, &prime) )
+  { ret = BN_is_prime_ex(prime, nchecks, ctx, NULL);
+  }
+
+  BN_free(prime);
+  BN_CTX_free(ctx);
+
+  if ( ret == -1 )
+  { return raise_ssl_error(ERR_get_error());
+  }
+
+  return ret;
+}
+
+
+                /*******************************
+                *        ELLIPTIC CURVES       *
+                *******************************/
+
+#ifndef OPENSSL_NO_EC
+
+#define CURVE_MAGIC (~ 0x51431485L)
+
+typedef struct curve
+{ int             magic;
+  atom_t          atom;
+
+  EC_GROUP       *group;
+  BN_CTX         *ctx;
+} PL_CRYPTO_CURVE;
+
+static int
+free_crypto_curve(PL_CRYPTO_CURVE *c)
+{ BN_CTX_free(c->ctx);
+  EC_GROUP_free(c->group);
+  free(c);
+  return TRUE;
+}
+
+static int
+release_curve(atom_t atom)
+{ size_t size;
+  PL_CRYPTO_CURVE **cp = PL_blob_data(atom, &size, NULL);
+  PL_CRYPTO_CURVE *c = *cp;
+  ssl_deb(4, "Releasing PL_CRYPTO_CURVE %p\n", c);
+  free_crypto_curve(c);
+  return TRUE;
+}
+
+static int
+compare_curve(atom_t a, atom_t b)
+{ PL_CRYPTO_CURVE**cp1 = PL_blob_data(a, NULL, NULL);
+  PL_CRYPTO_CURVE**cp2 = PL_blob_data(b, NULL, NULL);
+  PL_CRYPTO_CURVE *c1 = *cp1;
+  PL_CRYPTO_CURVE *c2 = *cp2;
+
+  return ( c1 > c2 ?  1 :
+           c1 < c2 ? -1 : 0
+         );
+}
+
+static int
+write_curve(IOSTREAM *s, atom_t symbol, int flags)
+{ PL_CRYPTO_CURVE **cp = PL_blob_data(symbol, NULL, NULL);
+  PL_CRYPTO_CURVE *c = *cp;
+  const char *name = OBJ_nid2sn(EC_GROUP_get_curve_name(c->group));
+
+  Sfprintf(s, "<crypto_curve>(%s, %p)", name, c);
+
+  return TRUE;
+}
+
+static void
+acquire_curve(atom_t atom)
+{ size_t size;
+  PL_CRYPTO_CURVE **cp = PL_blob_data(atom, &size, NULL);
+  PL_CRYPTO_CURVE *c = *cp;
+  c->atom = atom;
+}
+
+static PL_blob_t crypto_curve_type =
+{ PL_BLOB_MAGIC,
+  0,
+  "crypto_curve",
+  release_curve,
+  compare_curve,
+  write_curve,
+  acquire_curve
+};
+
+static int
+unify_curve(term_t tcurve, PL_CRYPTO_CURVE *curve)
+{ if ( PL_unify_blob(tcurve, &curve, sizeof(curve), &crypto_curve_type) )
+    return TRUE;
+
+  free_crypto_curve(curve);
+
+  if ( !PL_exception(0) )
+    return PL_uninstantiation_error(tcurve);
+
   return FALSE;
 }
 
+static int
+get_curve(term_t tcurve, PL_CRYPTO_CURVE **curve)
+{ PL_blob_t *type;
+  void *data;
+
+  if ( PL_get_blob(tcurve, &data, NULL, &type) &&
+       type == &crypto_curve_type )
+  { PL_CRYPTO_CURVE *c = *(PL_CRYPTO_CURVE**)data;
+
+    assert(c->magic == CURVE_MAGIC);
+    *curve = c;
+
+    return TRUE;
+  }
+
+  return PL_type_error("crypto_curve", tcurve);
+}
+
+#endif
+
+static foreign_t
+pl_crypto_name_curve(term_t tname, term_t tcurve)
+{
+#ifndef OPENSSL_NO_EC
+  PL_CRYPTO_CURVE *curve = NULL;
+  char *name;
+
+  if ( !PL_get_chars(tname, &name, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+    return FALSE;
+
+  if ( !(curve = malloc(sizeof(*curve))) )
+    return PL_resource_error("memory");
+
+  curve->magic    = CURVE_MAGIC;
+  curve->ctx      = NULL;
+  curve->group    = NULL;
+
+  if ( ( curve->group = EC_GROUP_new_by_curve_name(OBJ_sn2nid(name)) ) &&
+       ( curve->ctx   = BN_CTX_new() ) )
+  { return unify_curve(tcurve, curve);
+  } else
+  { BN_CTX_free(curve->ctx);
+    EC_GROUP_free(curve->group);
+    free(curve);
+
+    return raise_ssl_error(ERR_get_error());
+  }
+#else
+  return ssl_missing("EC");
+#endif
+}
+
+static foreign_t
+pl_crypto_curve_order(term_t tcurve, term_t torder)
+{
+#ifndef OPENSSL_NO_EC
+  PL_CRYPTO_CURVE *curve = NULL;
+  BIGNUM *order = NULL;
+  char *hex = NULL;
+  int rc = FALSE, ssl_err = FALSE;
+
+  if ( !get_curve(tcurve, &curve) )
+    return FALSE;
+
+  if ( ( order = BN_new() ) &&
+       EC_GROUP_get_order(curve->group, order, curve->ctx) &&
+       ( hex = BN_bn2hex(order) ) )
+  { rc = PL_unify_chars(torder, PL_STRING|REP_ISO_LATIN_1, strlen(hex), hex);
+  } else
+  { ssl_err = TRUE;
+  }
+
+  OPENSSL_free(hex);
+  BN_free(order);
+
+  if ( ssl_err )
+  { return raise_ssl_error(ERR_get_error());
+  }
+
+  return rc;
+#else
+  return ssl_missing("EC");
+#endif
+}
+
+
+static foreign_t
+pl_crypto_curve_generator(term_t tcurve, term_t tx, term_t ty)
+{
+#ifndef OPENSSL_NO_EC
+  PL_CRYPTO_CURVE *curve = NULL;
+  BIGNUM *x = NULL, *y = NULL;
+  char *xhex = NULL, *yhex = NULL;
+  int rc = FALSE, ssl_err = FALSE;
+
+  if ( !get_curve(tcurve, &curve) )
+    return FALSE;
+
+  if ( ( x = BN_new() ) &&
+       ( y = BN_new() ) &&
+       EC_POINT_get_affine_coordinates_GFp(curve->group,
+                                           EC_GROUP_get0_generator(curve->group),
+                                           x, y, curve->ctx) &&
+       ( xhex = BN_bn2hex(x) ) &&
+       ( yhex = BN_bn2hex(y) ) )
+  { rc = PL_unify_chars(tx, PL_STRING|REP_ISO_LATIN_1, strlen(xhex), xhex)
+      && PL_unify_chars(ty, PL_STRING|REP_ISO_LATIN_1, strlen(yhex), yhex);
+  } else
+  { ssl_err = TRUE;
+  }
+
+  OPENSSL_free(xhex); OPENSSL_free(yhex);
+  BN_free(x); BN_free(y);
+
+  if ( ssl_err )
+  { return raise_ssl_error(ERR_get_error());
+  }
+
+  return rc;
+#else
+  return ssl_missing("EC");
+#endif
+}
+
+
+
+static foreign_t
+pl_crypto_curve_scalar_mult(term_t tcurve, term_t ts,
+                           term_t tx, term_t ty, term_t ta, term_t tb)
+{
+#ifndef OPENSSL_NO_EC
+  BIGNUM *s = NULL, *x = NULL, *y = NULL, *a = NULL, *b = NULL;
+  EC_POINT *r = NULL, *q = NULL;
+  char *ahex = NULL, *bhex = NULL;
+  PL_CRYPTO_CURVE *curve = NULL;
+  int rc, ssl_err = FALSE;
+
+  if ( !get_curve(tcurve, &curve) )
+    return FALSE;
+
+  if ( get_bn_arg(1, ts, &s) &&
+       get_bn_arg(1, tx, &x) &&
+       get_bn_arg(1, ty, &y)  &&
+       ( q = EC_POINT_new(curve->group) ) &&
+       EC_POINT_set_affine_coordinates_GFp(curve->group, q, x, y, curve->ctx) &&
+       ( r = EC_POINT_new(curve->group) ) &&
+       EC_POINT_mul(curve->group, r, NULL, q, s, curve->ctx) &&
+       ( a = BN_new() ) &&
+       ( b = BN_new() ) &&
+       EC_POINT_get_affine_coordinates_GFp(curve->group, r, a, b, curve->ctx) &&
+       ( ahex = BN_bn2hex(a) ) &&
+       ( bhex = BN_bn2hex(b) ) )
+  { rc = PL_unify_chars(ta, PL_STRING|REP_ISO_LATIN_1, strlen(ahex), ahex)
+      && PL_unify_chars(tb, PL_STRING|REP_ISO_LATIN_1, strlen(bhex), bhex);
+  } else
+  { rc = FALSE;					/* silence compiler */
+    ssl_err = TRUE;
+  }
+
+  OPENSSL_free(ahex); OPENSSL_free(bhex);
+  BN_free(a); BN_free(b);
+  BN_free(s); BN_free(x); BN_free(y);
+  EC_POINT_free(q); EC_POINT_free(r);
+
+  if ( ssl_err )
+    return raise_ssl_error(ERR_get_error());
+
+  return rc;
+#else
+  return ssl_missing("EC");
+#endif
+}
 
 
                 /*******************************
@@ -1322,6 +1864,9 @@ static int
 crypto_lib_init(void)
 {
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
+  OpenSSL_add_all_algorithms();
+  ERR_load_crypto_strings();
+
   if ( (old_id_callback=CRYPTO_THREADID_get_callback()) == 0 )
   { int i;
 
@@ -1401,6 +1946,11 @@ install_crypto4pl(void)
   MKATOM(md5);
   MKATOM(blake2b512);
   MKATOM(blake2s256);
+  MKATOM(sha3_224);
+  MKATOM(sha3_256);
+  MKATOM(sha3_384);
+  MKATOM(sha3_512);
+  MKATOM(ripemd160);
 
   MKATOM(pkcs1);
   MKATOM(pkcs1_oaep);
@@ -1415,13 +1965,24 @@ install_crypto4pl(void)
   FUNCTOR_public_key1       = PL_new_functor(PL_new_atom("public_key"), 1);
   FUNCTOR_private_key1      = PL_new_functor(PL_new_atom("private_key"), 1);
 
-  PL_register_foreign("crypto_context_new", 2, pl_crypto_context_new, 0);
-  PL_register_foreign("_crypto_update_context", 2, pl_crypto_update_context, 0);
-  PL_register_foreign("_crypto_context_copy", 2, pl_crypto_context_copy, 0);
-  PL_register_foreign("_crypto_context_hash", 2, pl_crypto_context_hash, 0);
+  PL_register_foreign("crypto_n_random_bytes", 2, pl_crypto_n_random_bytes, 0);
+
+  PL_register_foreign("_crypto_context_new", 2,
+                      pl_crypto_hash_context_new, 0);
+  PL_register_foreign("_crypto_update_hash_context", 2,
+                      pl_crypto_update_hash_context, 0);
+  PL_register_foreign("_crypto_hash_context_copy", 2,
+                      pl_crypto_hash_context_copy, 0);
+  PL_register_foreign("_crypto_hash_context_hash", 2,
+                      pl_crypto_hash_context_hash, 0);
+
   PL_register_foreign("_crypto_open_hash_stream", 3,
                       pl_crypto_open_hash_stream, 0);
-  PL_register_foreign("_crypto_stream_context", 2, pl_crypto_stream_context, 0);
+  PL_register_foreign("_crypto_stream_hash_context", 2,
+                      pl_crypto_stream_hash_context, 0);
+
+  PL_register_foreign("_crypto_password_hash", 4, pl_crypto_password_hash, 0);
+  PL_register_foreign("_crypto_data_hkdf", 7, pl_crypto_data_hkdf, 0);
 
   PL_register_foreign("_crypto_ecdsa_sign", 4, pl_ecdsa_sign, 0);
   PL_register_foreign("_crypto_ecdsa_verify", 4, pl_ecdsa_verify, 0);
@@ -1432,8 +1993,21 @@ install_crypto4pl(void)
   PL_register_foreign("rsa_public_encrypt", 4, pl_rsa_public_encrypt, 0);
   PL_register_foreign("rsa_sign", 5, pl_rsa_sign, 0);
   PL_register_foreign("rsa_verify", 5, pl_rsa_verify, 0);
-  PL_register_foreign("evp_decrypt", 6, pl_evp_decrypt, 0);
-  PL_register_foreign("evp_encrypt", 6, pl_evp_encrypt, 0);
+  PL_register_foreign("_crypto_data_decrypt", 7, pl_crypto_data_decrypt, 0);
+  PL_register_foreign("_crypto_data_encrypt", 8, pl_crypto_data_encrypt, 0);
+
+  PL_register_foreign("_crypto_modular_inverse", 3,
+                      pl_crypto_modular_inverse, 0);
+  PL_register_foreign("_crypto_generate_prime", 4,
+                      pl_crypto_generate_prime, 0);
+  PL_register_foreign("_crypto_is_prime", 2, pl_crypto_is_prime, 0);
+
+  PL_register_foreign("crypto_name_curve", 2, pl_crypto_name_curve, 0);
+  PL_register_foreign("_crypto_curve_order", 2, pl_crypto_curve_order, 0);
+  PL_register_foreign("_crypto_curve_generator", 3,
+                      pl_crypto_curve_generator, 0);
+  PL_register_foreign("_crypto_curve_scalar_mult", 6,
+                      pl_crypto_curve_scalar_mult, 0);
 
   /*
    * Initialize crypto library
