@@ -36,8 +36,12 @@
 */
 
 #include "md.h"
-#include <gnu/system.h>
+#include <time.h>
+#ifdef HAVE_SYS_TIMEB_H
+#include <sys/timeb.h>
+#endif
 #include <ctype.h>
+#include <stdlib.h>
 
 #define malloc(n)	pceMalloc(n)
 #define free(p)		pceFree(p)
@@ -80,8 +84,6 @@ extern void *pceFree(void *p);
    host does not conform to Posix.  */
 #define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)
 
-#include "getdate.h"
-
 #if defined (STDC_HEADERS) || defined (USG)
 # include <string.h>
 #endif
@@ -432,7 +434,7 @@ void *alloca ();
 #define YYEMPTY		-2
 #define YYEOF		0
 #define YYACCEPT	return(0)
-#define YYABORT 	return(1)
+#define YYABORT		return(1)
 #define YYERROR		goto yyerrlab1
 /* Like YYERROR except do call yyerror.
    This remains here temporarily to ease the
@@ -1206,8 +1208,8 @@ case 48:
 		      }
 		    else
 		      {
-		    	yyHour = yyvsp[0].Number / 100;
-		    	yyMinutes = yyvsp[0].Number % 100;
+			yyHour = yyvsp[0].Number / 100;
+			yyMinutes = yyvsp[0].Number % 100;
 		      }
 		    yySeconds = 0;
 		    yyMeridian = MER24;
