@@ -1,25 +1,35 @@
-/*  $Id$
-
-    Part of XPCE --- The SWI-Prolog GUI toolkit
+/*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org/packages/xpce/
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (c)  2011-2015, University of Amsterdam
+    All rights reserved.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions
+    are met:
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+    1. Redistributions of source code must retain the above copyright
+       notice, this list of conditions and the following disclaimer.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    2. Redistributions in binary form must reproduce the above copyright
+       notice, this list of conditions and the following disclaimer in
+       the documentation and/or other materials provided with the
+       distribution.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+    POSSIBILITY OF SUCH DAMAGE.
 */
 
 #ifdef __WINDOWS__
@@ -434,7 +444,7 @@ GetChars(term_t t, char **s, unsigned int *len)
 static int
 StripModuleTag(term_t t, atom_t *module, term_t p)
 { atom_t name;
-  int arity;
+  size_t arity;
   term_t a = PL_new_term_ref();
 
   PutTerm(p, t);
@@ -955,7 +965,7 @@ PceName	atomToName(atom_t a)
 	XPCE names and Prolog atoms.
 
 PceObject referenceToObject(term_t a)
-	a is the argment to @/1.  Translate to an XPCE object or raise
+	a is the argument to @/1.  Translate to an XPCE object or raise
 	an error.  This function too caches using the
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -1485,7 +1495,7 @@ termToObject(term_t t, PceType targettype, atom_t assoc, int new)
 static PceObject
 termToObject(term_t t, PceType type, atom_t assoc, int new)
 { atom_t functor;
-  int arity;
+  size_t arity;
 
   DEBUG(Sdprintf("termToObject(");
 	PL_write_term(Soutput, t, 1200, 0);
@@ -1688,7 +1698,7 @@ unifyObject(term_t t, PceObject obj, int top)
     case PCE_ASSOC:
       if ( !top )
       { atom_t n;
-	int a;
+	size_t a;
 
 	if ( IsVar(t) )			/* get(R, S, Var) */
 	  return unifyReference(t, pcetype, value);
@@ -1724,7 +1734,7 @@ unifyObject(term_t t, PceObject obj, int top)
   }
 
   { atom_t name;
-    int n, arity;
+    size_t n, arity;
     atom_t pname;				/* name of Pce object */
     int parity;				/* its `arity' */
     PceObject got;			/* temp variable */
@@ -1992,7 +2002,7 @@ invoke(term_t rec, term_t cl, term_t msg, term_t ret)
 
   if ( receiver )
   { atom_t name;
-    int  arity;
+    size_t  arity;
     PceClass class;
 
     if ( !get_pce_class(cl, &class) )
@@ -2217,7 +2227,7 @@ pl_get_class(term_t rec, term_t cl, term_t msg, term_t ret)
 static foreign_t
 pl_object1(term_t ref)
 { atom_t name;
-  int arity;
+  size_t arity;
 
   if ( GetNameArity(ref, &name, &arity) &&
        name == ATOM_ref &&
@@ -3110,7 +3120,7 @@ unify_prof_node(term_t t, void *impl)
 static int
 get_prof_node(term_t ref, void **impl)
 { atom_t name;
-  int arity;
+  size_t arity;
 
   if ( GetNameArity(ref, &name, &arity) &&
        name == ATOM_ref &&
