@@ -468,13 +468,13 @@ retry:
     fr->programPointer = pcret;
     fr->parent         = pfr;
     fr->clause         = cref;
-    fr->predicate      = cl->predicate;
+    setFramePredicate(fr, cl->predicate);
     fr->context	       = fr->predicate->module;
     setNextFrameFlags(fr, pfr);
 #ifdef O_PROFILE
     fr->prof_node      = NULL;
 #endif
-    setGenerationFrame(fr, global_generation());
+    setGenerationFrame(fr);
     enterDefinition(fr->predicate);
     environment_frame = fr;
 
