@@ -101,6 +101,12 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_READ_TOKEN),
 
   DEBUG_TOPIC(MSG_CONTINUE),
+
+  DEBUG_TOPIC(MSG_CLEANUP_THREAD),
+  DEBUG_TOPIC(MSG_INITIALISE),
+
+  DEBUG_TOPIC(MSG_BACKTRACK),
+
 						/* GC messages */
   DEBUG_TOPIC(MSG_AGC),
   DEBUG_TOPIC(MSG_CLAUSE_GC),
@@ -133,6 +139,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_ATTVAR_LINK),
   DEBUG_TOPIC(MSG_CALL_RESIDUE_VARS),
   DEBUG_TOPIC(MSG_SOFTCUT),
+  DEBUG_TOPIC(MSG_WAKEUP),
 
   DEBUG_TOPIC(MSG_HASH_TABLE_API),
   DEBUG_TOPIC(MSG_HASH_TABLE_KVS),
