@@ -94,6 +94,7 @@ Prolog int) is used by the garbage collector to update the stack frames.
 	       return raiseStackOverflow(GLOBAL_OVERFLOW); } while(0)
 
 static int	unify_int64_ex__LD(term_t t, int64_t i, int ex ARG_LD);
+static int	PL_get_uint__LD(term_t t, unsigned int *i ARG_LD);
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Deduce the value to store a copy of the  contents of p. This is a *very*
@@ -818,17 +819,125 @@ PL_cvt_set_encoding(int enc)
 
 #define REP_SP (sp_encoding)
 
+#ifndef SCHAR_MIN
+#define SCHAR_MIN -128
+#define SCHAR_MAX 127
+#endif
+#ifndef UCHAR_MAX
+#define UCHAR_MAX 255
+#endif
+
+#ifndef SHORT_MIN
+#define SHORT_MIN -32768
+#define SHORT_MAX 32767
+#define USHORT_MAX (SHORT_MAX*2+1)
+#endif
+
+static bool
+_PL_cvt_i_char(term_t p, char *c, int mn, int mx)
+{ GET_LD
+  int i;
+  PL_chars_t txt;
+
+  if ( PL_get_integer(p, &i) && i >= mn && i <= mx )
+  { *c = (char)i;
+    return TRUE;
+  } else if ( PL_get_text(p, &txt, CVT_ATOM|CVT_STRING|CVT_LIST) )
+  { if ( txt.length == 1 && txt.encoding == ENC_ISO_LATIN_1 )
+    { *c = txt.text.t[0];
+      return TRUE;			/* can never be allocated */
+    }
+    PL_free_text(&txt);
+  }
+
+  if ( PL_is_integer(p) )
+    return PL_representation_error(mn < 0 ? "char" : "uchar");
+
+  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, p);
+}
+
+
+bool
+PL_cvt_i_char(term_t p, char *c)
+{ return _PL_cvt_i_char(p, c, SCHAR_MIN, SCHAR_MAX);
+}
+
+bool
+PL_cvt_i_uchar(term_t p, unsigned char *c)
+{ return _PL_cvt_i_char(p, (char *)c, 0, UCHAR_MAX);
+}
+
+
+static bool
+_PL_cvt_i_short(term_t p, short *s, int mn, int mx)
+{ GET_LD
+  int i;
+
+  if ( PL_get_integer(p, &i) &&
+       i >= mn && i <= mx )
+  { *s = (short)i;
+    return TRUE;
+  }
+
+  if ( PL_is_integer(p) )
+    return PL_representation_error(mn < 0 ? "short" : "ushort");
+
+  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, p);
+}
+
+bool
+PL_cvt_i_short(term_t p, short *s)
+{ return _PL_cvt_i_short(p, s, SHORT_MIN, SHORT_MAX);
+}
+
+bool
+PL_cvt_i_ushort(term_t p, unsigned short *s)
+{ return _PL_cvt_i_short(p, (short *)s, 0, USHORT_MAX);
+}
+
 bool
 PL_cvt_i_int(term_t p, int *c)
 { return PL_get_integer_ex(p, c);
 }
 
+bool
+PL_cvt_i_uint(term_t t, unsigned int *c)
+{ GET_LD
+
+  if ( PL_get_uint__LD(t, c PASS_LD) )
+    return TRUE;
+
+  if ( PL_is_integer(t) )
+    return PL_representation_error("uint");
+
+  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t);
+}
 
 bool
 PL_cvt_i_long(term_t p, long *c)
 { return PL_get_long_ex(p, c);
 }
 
+bool
+PL_cvt_i_ulong(term_t p, unsigned long *c)
+{
+#if SIZEOF_LONG == 8
+  return PL_cvt_i_uint64(p, c);
+#else
+  return PL_cvt_i_uint(p, (unsigned int*)c);
+#endif
+}
+
+bool
+PL_cvt_i_int64(term_t p, int64_t *c)
+{ return PL_get_int64_ex(p, c);
+}
+
+bool
+PL_cvt_i_uint64(term_t p, uint64_t *c)
+{ GET_LD
+  return PL_get_uint64_ex__LD(p, c PASS_LD);
+}
 
 bool
 PL_cvt_i_size_t(term_t p, size_t *c)
@@ -1443,6 +1552,7 @@ PL_get_integer__LD(term_t t, int *i ARG_LD)
     *i = (int)val;
     succeed;
   }
+#if SIZEOF_VOIDP < 8
   if ( isBignum(w) )
   { int64_t val = valBignum(w);
 
@@ -1452,6 +1562,7 @@ PL_get_integer__LD(term_t t, int *i ARG_LD)
     *i = (int)val;
     succeed;
   }
+#endif
   if ( isFloat(w) )
   { double f = valFloat(w);
     int l;
@@ -1480,6 +1591,50 @@ PL_get_integer(term_t t, int *i)
 #define PL_get_integer(t, i) PL_get_integer__LD(t, i PASS_LD)
 
 
+static int
+PL_get_uint__LD(term_t t, unsigned int *i ARG_LD)
+{ word w = valHandle(t);
+
+  if ( isTaggedInt(w) )
+  { intptr_t val = valInt(w);
+
+    if ( val < 0 || val > UINT_MAX )
+      fail;
+    *i = (unsigned int)val;
+    succeed;
+  }
+#if SIZEOF_VOIDP < 8
+  if ( isBignum(w) )
+  { int64_t val = valBignum(w);
+
+    if ( val < 0 || val > UINT_MAX )
+      fail;
+
+    *i = (unsigned int)val;
+    succeed;
+  }
+#endif
+#ifndef O_GMP
+  if ( isFloat(w) )
+  { double f = valFloat(w);
+    unsigned int l;
+
+#ifdef DOUBLE_TO_LONG_CAST_RAISES_SIGFPE
+    if ( f > (double)UINT_MAX || f < 0.0 )
+      fail;
+#endif
+
+    l = (unsigned int)f;
+    if ( (double)l == f )
+    { *i = l;
+      succeed;
+    }
+  }
+#endif
+  fail;
+}
+
+
 int
 PL_get_long__LD(term_t t, long *i ARG_LD)
 { word w = valHandle(t);
