@@ -327,6 +327,7 @@ COMMON(void)		clearLocalVariablesFrame(LocalFrame fr);
 COMMON(void)		setLTopInBody(void);
 COMMON(word)		check_foreign(void);	/* DEBUG(CHK_SECURE...) stuff */
 COMMON(void)		markAtomsOnStacks(PL_local_data_t *ld);
+COMMON(void)		set_min_generation(DirtyDefInfo ddi, gen_t gen);
 COMMON(void)		markPredicatesInEnvironments(PL_local_data_t *ld);
 COMMON(QueryFrame)	queryOfFrame(LocalFrame fr);
 COMMON(void)		mark_active_environment(struct bit_vector *active,
@@ -502,7 +503,6 @@ COMMON(void)		unallocClause(Clause c);
 COMMON(void)		freeClause(Clause c);
 COMMON(void)		lingerClauseRef(ClauseRef c);
 COMMON(ClauseRef)	newClauseRef(Clause cl, word key);
-COMMON(void)		gcClauseRefs(void);
 COMMON(size_t)		removeClausesPredicate(Definition def,
 					       int sfindex, int fromfile);
 COMMON(void)		reconsultFinalizePredicate(sf_reload *rl, Definition def,
