@@ -985,19 +985,18 @@ typedef struct lgen_t
 
 #define generationFrame(f) \
 	((gen_t)(f)->generation.gen_u<<32 | (gen_t)(f)->generation.gen_l)
-#define setGenerationFrame(f, gen) \
-	do { (f)->generation.gen_u = (uint32_t)(gen>>32); \
-	     (f)->generation.gen_l = (uint32_t)(gen); \
+#define setGenerationFrameVal(f, g) \
+	do { gen_t __gen = (g); \
+	     (f)->generation.gen_u = (uint32_t)(__gen>>32); \
+	     (f)->generation.gen_l = (uint32_t)(__gen); \
 	   } while(0)
 #else
 typedef uint64_t lgen_t;
 #define generationFrame(f)	((f)->generation)
-#define setGenerationFrame(f, gen) \
+#define setGenerationFrameVal(f, gen) \
 	do { (f)->generation = (gen); } while(0)
 #endif
 #ifdef HAVE___SYNC_ADD_AND_FETCH_8
-#define global_generation() (GD->_generation)
-#define next_global_generation() ATOMIC_INC(&GD->_generation)
 typedef uint64_t ggen_t;
 #else
 #define ATOMIC_GENERATION_HACK 1
@@ -1007,12 +1006,12 @@ typedef struct ggen_t
 } ggen_t;
 #endif /*HAVE___SYNC_ADD_AND_FETCH_8*/
 #else /*O_LOGICAL_UPDATE*/
-#define global_generation()	(0)
+#define global_generation()	 (0)
 #define next_global_generation() (0)
-#define generationFrame(f)	(0)
-#define setGenerationFrame(f)	(void)0
 #endif /*O_LOGICAL_UPDATE*/
 
+#define setGenerationFrame(fr) setGenerationFrame__LD((fr) PASS_LD)
+
 #define FR_CLEAR_NEXT	FR_SKIPPED|FR_WATCHED|FR_CATCHED|FR_HIDE_CHILDS|FR_CLEANUP
 #define FR_CLEAR_FLAGS	(FR_CLEAR_NEXT|FR_CONTEXT)
 
@@ -1022,6 +1021,11 @@ typedef struct ggen_t
 	  (next)->flags = ((fr)->flags) & ~FR_CLEAR_FLAGS; \
 	} while(0)
 
+#define setFramePredicate(fr, def) \
+	do \
+	{ (fr)->predicate = (def); \
+	} while(0)
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Predicate reference counting. The aim  of   this  mechanism  is to avoid
 modifying the predicate structure while  it   has  choicepoints  or (MT)
@@ -1154,15 +1158,13 @@ typedef struct atom_table
 #define ATOM_TYPE_INVALID	((PL_blob_t*)0x007)
 
 #ifdef O_DEBUG_ATOMGC
+extern IOSTREAM *atomLogFd;
 #define PL_register_atom(a) \
 	_PL_debug_register_atom(a, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define PL_unregister_atom(a) \
 	_PL_debug_unregister_atom(a, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#undef atomValue
-#define atomValue(a) _PL_debug_atom_value(a)
-extern Atom _PL_debug_atom_value(atom_t a);
 #endif
-#else
+#else /*!O_ATOMGC*/
 #define PL_register_atom(a)
 #define PL_unregister_atom(a)
 #endif
@@ -1455,8 +1457,18 @@ struct clause_choice
 };
 
 #ifdef O_PLMT
-#define acquire_def(def) do { LD->thread.info->access.predicate = def; } while(0)
-#define release_def(def) do { LD->thread.info->access.predicate = NULL; } while(0)
+#define acquire_def(def) \
+	do { DEBUG(CHK_SECURE, assert(!LD->thread.info->access.predicate)); \
+	     LD->thread.info->access.predicate = def; } while(0)
+#define release_def(def) \
+	do { LD->thread.info->access.predicate = NULL; } while(0)
+#define acquire_def2(def, store) \
+	do { store = LD->thread.info->access.predicate; \
+	     DEBUG(CHK_SECURE, assert(!store || store == def)); \
+	     LD->thread.info->access.predicate = def; } while(0)
+#define release_def2(def, store) \
+	do { LD->thread.info->access.predicate = store; } while(0)
+
 #else
 #define acquire_def(def) (void)0
 #define release_def(def) (void)0
