@@ -455,11 +455,12 @@ nextClause__LD(ClauseChoice chp, Word argv, LocalFrame fr, Definition def ARG_LD
   } else
   { cref = nextClauseArg1(chp, generation PASS_LD);
   }
-  DEBUG(CHK_SECURE, assert(!cref || !chp->cref ||
-			   visibleClause(chp->cref->value.clause,
-					 generationFrame(fr))));
   release_def();
 
+  DEBUG(CHK_SECURE,
+	assert(!cref || !chp->cref ||
+	       chp->cref->value.clause->generation.erased > generation));
+
   return cref;
 }
 
@@ -1349,7 +1350,9 @@ hashDefinition(Definition def, hash_hints *hints)
 
     ci = newClauseIndexTable(hints->args, hints);
 
-    acquire_def(def);
+#ifdef O_PLMT
+    assert(LD->thread.info->access.predicate == def);
+#endif
     first = def->impl.clauses.first_clause;
     last  = def->impl.clauses.last_clause;
 
@@ -1357,7 +1360,6 @@ hashDefinition(Definition def, hash_hints *hints)
     { if ( false(cref->value.clause, CL_ERASED) )
 	addClauseToIndex(ci, cref->value.clause, CL_END);
     }
-    release_def(def);
 
     LOCKDEF(def);
     if ( first == def->impl.clauses.first_clause &&
@@ -1636,8 +1638,9 @@ checkClauseIndexes(Definition def)
       for(i=0,cb=ci->entries; i<ci->buckets; i++,cb++)
       { ClauseRef cref;
 	unsigned int dirty = 0;
+	Definition old;
 
-	acquire_def(def);
+	acquire_def2(def, old);
 	for(cref=cb->head; cref; cref=cref->next)
 	{ if ( cref->d.key )
 	    ci_size++;
@@ -1665,7 +1668,7 @@ checkClauseIndexes(Definition def)
 	      dirty++;
 	  }
 	}
-	release_def(def);
+	release_def2(def, old);
 
 	assert(cb->dirty == dirty);
 	if ( cb->dirty )
