@@ -52,9 +52,6 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include <process.h>			/* getpid() */
 #endif
 
-#define LOCK()   PL_LOCK(L_INIT)
-#define UNLOCK() PL_UNLOCK(L_INIT)
-
 #if defined(_DEBUG) && defined(__WINDOWS__) && !defined(__MINGW32__)
 #include <crtdbg.h>
 #endif
@@ -348,14 +345,14 @@ initPaths(int argc, const char **argv)
 	 !(symbols = DeRefLink(symbols, plp)) )
       symbols = argv[0];
 
-    DEBUG(2, Sdprintf("rc-module: %s\n", symbols));
+    DEBUG(MSG_INITIALISE, Sdprintf("rc-module: %s\n", symbols));
 
     systemDefaults.home	       = findHome(symbols, argc, argv);
 
 #ifdef __WINDOWS__			/* we want no module but the .EXE */
     GD->paths.module	       = store_string(symbols);
     symbols = findExecutable(NULL, plp, sizeof(plp));
-    DEBUG(2, Sdprintf("Executable: %s\n", symbols));
+    DEBUG(MSG_INITIALISE, Sdprintf("Executable: %s\n", symbols));
 #endif
     GD->paths.executable       = store_string(symbols);
     GD->options.systemInitFile = defaultSystemInitFile(argv[0]);
@@ -436,7 +433,7 @@ initDefaults(void)
   systemDefaults.trail       = DEFTRAIL;
   systemDefaults.table       = DEFTABLE;
   systemDefaults.goal	     = NULL;
-  systemDefaults.toplevel    = "prolog";
+  systemDefaults.toplevel    = "default";
   systemDefaults.notty       = NOTTYCONTROL;
 
 #ifdef __WINDOWS__
@@ -446,13 +443,16 @@ initDefaults(void)
   GD->io_initialised	     = FALSE;
   GD->initialised	     = FALSE;
   GD->bootsession	     = FALSE;
+#ifdef SIG_ALERT
+  GD->signals.sig_alert      = SIG_ALERT;
+#endif
 
   if ( systemDefaults.notty )
     clearPrologFlagMask(PLFLAG_TTY_CONTROL);
   else
     setPrologFlagMask(PLFLAG_TTY_CONTROL);
 
-  setPrologFlagMask(PLFLAG_DEBUGINFO);
+  setPrologFlagMask(PLFLAG_DEBUGINFO|PLFLAG_GCTHREAD);
 }
 
 
@@ -568,6 +568,12 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
 { GET_LD
   int argc = argc0;
 
+  DEBUG(MSG_INITIALISE,
+	{ Sdprintf("parseCommandLineOptions");
+	  for(int i=0; i<argc; i++)
+	    Sdprintf("%s ", argv[i]);
+	});
+
   for( ; argc > 0 && (argv[0][0] == '-' || argv[0][0] == '+'); argc--, argv++ )
   { char *s = &argv[0][1];
 
@@ -584,6 +590,7 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
       continue;
     } else if ( isoption(s, "nosignals") )
     { clearPrologFlagMask(PLFLAG_SIGNALS);
+      clearPrologFlagMask(PLFLAG_GCTHREAD);
       continue;
     } else if ( isoption(s, "nodebug") )
     { clearPrologFlagMask(PLFLAG_DEBUGINFO);
@@ -608,6 +615,14 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
 #endif
       } else if ( (optval=is_longopt(s, "traditional")) )
       { setTraditional();
+      } else if ( (optval=is_longopt(s, "sigalert")) )
+      { char *e;
+	long sig = strtol(optval, &e, 10);
+
+	if ( e > optval && *e == EOS && sig >= 0 && sig < 32 )
+	  GD->signals.sig_alert = sig;
+	else
+	  return -1;
       }
 
       continue;				/* don't handle --long=value */
@@ -816,7 +831,7 @@ PL_initialise(int argc, char **argv)
 
   { GET_LD
 #ifdef HAVE_SIGNAL
-  setPrologFlagMask(PLFLAG_SIGNALS);	/* default: handle signals */
+    setPrologFlagMask(PLFLAG_SIGNALS);	/* default: handle signals */
 #endif
 
   if ( !GD->resourceDB )
@@ -849,7 +864,7 @@ PL_initialise(int argc, char **argv)
       }
     }
 
-    DEBUG(1, if (GD->bootsession) Sdprintf("Boot session\n"););
+    DEBUG(MSG_INITIALISE, if (GD->bootsession) Sdprintf("Boot session\n"););
 
     if ( !GD->resourceDB )
     { if ( !(GD->resourceDB = openResourceDB(argc, argv)) )
@@ -915,8 +930,7 @@ PL_initialise(int argc, char **argv)
     if ( statefd )
     { GD->bootsession = TRUE;
       if ( !loadWicFromStream(statefd) )
-      { fail;
-      }
+	return FALSE;
       GD->bootsession = FALSE;
 
       Sclose(statefd);
@@ -933,7 +947,7 @@ PL_initialise(int argc, char **argv)
   GD->initialised = TRUE;
   registerForeignLicenses();
 
-  DEBUG(1, Sdprintf("Starting Prolog Part of initialisation\n"));
+  DEBUG(MSG_INITIALISE, Sdprintf("Starting Prolog Part of initialisation\n"));
 
   if ( compile )
   { int status = prologToplevel(PL_new_atom("$compile")) ? 0 : 1;
@@ -1220,9 +1234,9 @@ cleanupProlog(int rval, int reclaim_memory)
     PlMessage("Exit status is %d", rval);
 #endif
 
-  LOCK();
+  PL_LOCK(L_INIT);
   if ( GD->cleaning != CLN_NORMAL )
-  { UNLOCK();
+  { PL_UNLOCK(L_INIT);
     return FALSE;
   }
 
@@ -1239,7 +1253,7 @@ cleanupProlog(int rval, int reclaim_memory)
   debugmode(FALSE, NULL);		/* avoid recursive tracing */
 
   if ( GD->initialised )
-  { DEBUG(5, Sdprintf("Running at_halt hooks\n"));
+  { DEBUG(MSG_CLEANUP, Sdprintf("Running at_halt hooks\n"));
 
     if ( LD->outofstack )
       emptyStacks();
@@ -1249,7 +1263,7 @@ cleanupProlog(int rval, int reclaim_memory)
 	 rval == 0 )
     { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
       { GD->cleaning = CLN_NORMAL;
-	UNLOCK();
+	PL_UNLOCK(L_INIT);
 	return FALSE;
       }
     }
@@ -1258,7 +1272,7 @@ cleanupProlog(int rval, int reclaim_memory)
     if ( !run_on_halt(&GD->os.on_halt_list, rval) && rval == 0 )
     { if ( ++GD->halt_cancelled	< MAX_HALT_CANCELLED )
       { GD->cleaning = CLN_NORMAL;
-	UNLOCK();
+	PL_UNLOCK(L_INIT);
 	return FALSE;
       }
     }
@@ -1338,7 +1352,7 @@ emergency:
     cleanupDebug();
   }
 
-  UNLOCK();				/* requires GD->thread.enabled */
+  PL_UNLOCK(L_INIT);				/* requires GD->thread.enabled */
 
   if ( reclaim_memory )
   { memset(&PL_global_data, 0, sizeof(PL_global_data));
@@ -1351,7 +1365,7 @@ emergency:
 
 int
 PL_cleanup(int rc)
-{ return cleanupProlog(rc, TRUE);
+{ return cleanupProlog(rc, rc == 0);
 }
 
 
