@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2014, University of Amsterdam
+    Copyright (c)  1985-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -34,8 +34,6 @@
 */
 
 #include "pl-incl.h"
-#define LOCK()   PL_LOCK(L_MODULE)
-#define UNLOCK() PL_UNLOCK(L_MODULE)
 #undef LD
 #define LD LOCAL_LD
 
@@ -124,18 +122,17 @@ lookupModule__LD(atom_t name ARG_LD)
   if ( (m = lookupHTable(GD->tables.modules, (void*)name)) )
     return m;
 
-  LOCK();
+  PL_LOCK(L_MODULE);
   m = _lookupModule(name PASS_LD);
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 
   return m;
 }
 
 
 Module
-isCurrentModule(atom_t name)
-{ GET_LD
-  return lookupHTable(GD->tables.modules, (void*)name);
+isCurrentModule__LD(atom_t name ARG_LD)
+{ return lookupHTable(GD->tables.modules, (void*)name);
 }
 
 
@@ -148,7 +145,7 @@ unallocModuleSymbol(void *name, void *value)
 void
 initModules(void)
 { GET_LD
-  LOCK();
+  PL_LOCK(L_MODULE);
   if ( !GD->tables.modules )
   {
 #ifdef O_PLMT
@@ -161,7 +158,7 @@ initModules(void)
     GD->modules.system = _lookupModule(ATOM_system PASS_LD);
     GD->modules.user   = _lookupModule(ATOM_user PASS_LD);
   }
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 }
 
 
@@ -399,9 +396,9 @@ int
 addSuperModule(Module m, Module s, int where)
 { int rc;
 
-  LOCK();
+  PL_LOCK(L_MODULE);
   rc = addSuperModule_no_lock(m, s, where);
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 
   return rc;
 }
@@ -443,9 +440,9 @@ clearSupersModule_no_lock(Module m)
 
 void
 clearSupersModule(Module m)
-{ LOCK();
+{ PL_LOCK(L_MODULE);
   clearSupersModule_no_lock(m);
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 }
 
 
@@ -491,9 +488,9 @@ PRED_IMPL("set_module", 1, set_module, PL_FA_TRANSPARENT)
       if ( !PL_get_atom_ex(arg, &mname) )
 	return FALSE;
       super = lookupModule(mname);
-      LOCK();
+      PL_LOCK(L_MODULE);
       rc = setSuperModule(m, super);
-      UNLOCK();
+      PL_UNLOCK(L_MODULE);
       return rc;
     } else if ( pname == ATOM_class )
     { atom_t class;
@@ -565,18 +562,17 @@ getUnknownModule(Module m)
   return u;
 }
 
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-stripModule() takes an atom or term, possible embedded in the :/2 module
-term.  It will assign *module with the associated module and return  the
-remaining term.
+stripModuleName() takes an atom or term,   possible  embedded in the :/2
+module term. It assigns *name  with   the  associated  module names. The
+return value is the plain term or NULL if `term` is cyclic.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 Word
-stripModule(Word term, Module *module ARG_LD)
+stripModuleName(Word term, atom_t *name ARG_LD)
 { int depth = 100;
   deRef(term);
+  atom_t nm = 0;
 
   while( hasFunctor(*term, FUNCTOR_colon2) )
   { Word mp;
@@ -584,7 +580,7 @@ stripModule(Word term, Module *module ARG_LD)
     deRef(mp);
     if ( !isTextAtom(*mp) )
       break;
-    *module = lookupModule(*mp);
+    nm = *mp;
     term = argTermP(*term, 1);
     deRef(term);
     if ( --depth == 0 && !is_acyclic(term PASS_LD) )
@@ -595,13 +591,46 @@ stripModule(Word term, Module *module ARG_LD)
     }
   }
 
-  if ( ! *module )
-    *module = (environment_frame ? contextModule(environment_frame)
-				 : MODULE_user);
+  if ( nm )
+    *name = nm;
 
   return term;
 }
 
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+stripModule() takes an atom or term, possible embedded in the :/2 module
+term.  It will assign *module with the associated module and return  the
+remaining term.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+Word
+stripModule(Word term, Module *module, int flags ARG_LD)
+{ atom_t mname = 0;
+  Word rc;
+
+  if ( (rc=stripModuleName(term, &mname PASS_LD)) )
+  { if ( mname )
+    { if ( unlikely(flags&SM_NOCREATE) )
+      { Module m;
+
+	if ( (m=isCurrentModule(mname)) )
+	  *module = m;
+	else
+	  return NULL;
+      } else
+      { *module = lookupModule(mname);
+      }
+    } else
+    { *module = (environment_frame ? contextModule(environment_frame)
+		                   : MODULE_user);
+    }
+  }
+
+  return rc;
+}
+
+
 bool
 isPublicModule(Module module, Procedure proc)
 { GET_LD
@@ -706,9 +735,9 @@ PRED_IMPL("delete_import_module", 2, delete_import_module, 0)
        !get_module(A2, &super, TRUE) )
     fail;
 
-  LOCK();
+  PL_LOCK(L_MODULE);
   rval = delSuperModule(me, super);
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 
   return rval;
 }
@@ -980,7 +1009,7 @@ find_modules_with_defs(Module m, int count, defm_target targets[],
 /** '$def_modules'(:list(PI), -list(Pair)) is det.
 
 Each Pair is a  pair  Module-list(PI),   where  Module:PI  is  a defined
-predicate in a the starting module or   a default module thereof. If the
+predicate in the starting module or  a   default  module thereof. If the
 first argument is qualified, this  is   the  starting  module. Else, the
 default source module is the starting module.   Only modules in which PI
 has a real definition are returned (i.e., _not_ modules where PI is only
@@ -1002,9 +1031,23 @@ PRED_IMPL("$def_modules", 2, def_modules, PL_FA_TRANSPARENT)
   term_t tmp   = PL_new_term_refs(3);
   term_t tail  = PL_copy_term_ref(A2);
   term_t thead = tmp+0;
+  atom_t mname = 0;
+  Word mp;
 
-  if ( !PL_strip_module(A1, &m, ttail) )
+  if ( !(mp=stripModuleName(valTermRef(A1), &mname PASS_LD)) )
     return FALSE;
+  *valTermRef(ttail) = linkVal(mp);
+
+  if ( mname )
+  { Module m2;
+
+    if ( (m2 = isCurrentModule(mname)) )
+      m = m2;
+    else if ( stringAtom(mname)[0] == '$' )
+      m = MODULE_system;
+    else
+      m = MODULE_user;
+  }
 
   while( PL_get_list_ex(ttail, thead, ttail) )
   { if ( tcount >= MAX_TARGETS )
@@ -1038,15 +1081,16 @@ declareModule(atom_t name, atom_t class, atom_t super,
 { GET_LD
   Module module = lookupModule(name);
   term_t tmp = 0, rdef = 0, rtail = 0;
+  int rc = TRUE;
 
-  LOCK();
+  PL_LOCK(L_MODULE);
   if ( class )
     module->class = class;
 
   if ( !allow_newfile && module->file && module->file != sf)
   { term_t obj;
     char msg[256];
-    UNLOCK();
+    PL_UNLOCK(L_MODULE);
 
     obj = PL_new_term_ref();
     PL_put_atom(obj, name);
@@ -1089,20 +1133,20 @@ declareModule(atom_t name, atom_t class, atom_t super,
   if ( super )
     setSuperModule(module, _lookupModule(super PASS_LD));
 
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 
   if ( rdef )
   { if ( !PL_unify_nil(rtail) )
       return FALSE;
 
-    printMessage(ATOM_warning,
-		 PL_FUNCTOR_CHARS, "declare_module", 2,
-		   PL_ATOM, name,
-		   PL_FUNCTOR_CHARS, "abolish", 1,
-		     PL_TERM, rdef);
+    rc = printMessage(ATOM_warning,
+		      PL_FUNCTOR_CHARS, "declare_module", 2,
+		        PL_ATOM, name,
+		        PL_FUNCTOR_CHARS, "abolish", 1,
+		          PL_TERM, rdef);
   }
 
-  succeed;
+  return rc;
 }
 
 
@@ -1171,31 +1215,38 @@ PRED_IMPL("$module_property", 2, module_property, 0)
 { PRED_LD
   Module m;
   term_t a = PL_new_term_ref();
+  atom_t pname;
+  size_t parity;
 
   if ( !get_module(A1, &m, FALSE) )
     fail;
 
-  if ( !PL_get_arg(1, A2, a) )
+  if ( !PL_get_name_arity(A2, &pname, &parity) ||
+       parity != 1 )
     return PL_error(NULL, 0, NULL, ERR_TYPE,
 		    ATOM_module_property, A2);
 
-  if ( PL_is_functor(A2, FUNCTOR_line_count1) )
+  _PL_get_arg(1, A2, a);
+
+  if ( pname == ATOM_line_count )
   { if ( m->line_no > 0 )
       return PL_unify_integer(a, m->line_no);
     else
       fail;
-  } else if ( PL_is_functor(A2, FUNCTOR_file1) )
+  } else if ( pname == ATOM_file )
   { if ( m->file )
       return PL_unify_atom(a, m->file->name);
     else
       fail;
-  } else if ( PL_is_functor(A2, FUNCTOR_exports1) )
+  } else if ( pname == ATOM_exports )
   { return unify_export_list(a, m PASS_LD);
-  } else if ( PL_is_functor(A2, FUNCTOR_class1) )
+  } else if ( pname == ATOM_class )
   { return PL_unify_atom(a, m->class);
-  } else if ( PL_is_functor(A2, FUNCTOR_program_size1) )
+  } else if ( pname == ATOM_program_size )
   { return PL_unify_int64(a, m->code_size);
-  } else if ( PL_is_functor(A2, FUNCTOR_program_space1) )
+  } else if ( pname == ATOM_last_modified_generation )
+  { return PL_unify_int64(a, m->last_modified);
+  } else if ( pname == ATOM_program_space )
   { if ( m->code_limit )
       return PL_unify_int64(a, m->code_limit);
     return FALSE;
@@ -1358,10 +1409,10 @@ fixExportModule(Module m, Definition old, Definition new)
 
 static void
 fixExport(Definition old, Definition new)
-{ LOCK();
+{ PL_LOCK(L_MODULE);
   for_table(GD->tables.modules, name, value,
 	    fixExportModule(value, old, new));
-  UNLOCK();
+  PL_UNLOCK(L_MODULE);
 }
 
 
@@ -1430,10 +1481,11 @@ retry:
 	  if ( !PL_unify_predicate(pi, proc, GP_NAMEARITY) )
 	    return FALSE;
 
-	  printMessage(ATOM_warning,
-		       PL_FUNCTOR_CHARS, "ignored_weak_import", 2,
-		         PL_ATOM, destination->name,
-		         PL_TERM, pi);
+	  if ( !printMessage(ATOM_warning,
+			     PL_FUNCTOR_CHARS, "ignored_weak_import", 2,
+			       PL_ATOM, destination->name,
+			       PL_TERM, pi) )
+	    return FALSE;
 	}
 
 	return TRUE;
@@ -1459,10 +1511,11 @@ retry:
 
     if ( !PL_unify_predicate(pi, proc, GP_NAMEARITY) )
       return FALSE;
-    printMessage(ATOM_warning,
-		 PL_FUNCTOR_CHARS, "import_private", 2,
-		   PL_ATOM, destination->name,
-		   PL_TERM, pi);
+    if ( !printMessage(ATOM_warning,
+		       PL_FUNCTOR_CHARS, "import_private", 2,
+		         PL_ATOM, destination->name,
+		         PL_TERM, pi) )
+      return FALSE;
   }
 
   { Procedure nproc = (Procedure)  allocHeapOrHalt(sizeof(struct procedure));
