@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2015, University of Amsterdam
+    Copyright (c)  1985-2017, University of Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -62,9 +62,6 @@ postfix). The best solution is probably to  define one structure for the
 whole thing.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define LOCK()   PL_LOCK(L_OP)
-#define UNLOCK() PL_UNLOCK(L_OP)
-
 static atom_t	operatorTypeToAtom(int type);
 
 		 /*******************************
@@ -157,14 +154,14 @@ defOperator(Module m, atom_t name, int type, int priority, int force)
   }
 
 
-  LOCK();
+  PL_LOCK(L_OP);
   if ( !m->operators )
     m->operators = newOperatorTable(8);
 
   if ( (op = lookupHTable(m->operators, (void *)name)) )
   { ;
   } else if ( priority < 0 )
-  { UNLOCK();				/* already inherited: do not change */
+  { PL_UNLOCK(L_OP);				/* already inherited: do not change */
     return TRUE;
   } else
   { op = allocHeapOrHalt(sizeof(*op));
@@ -185,7 +182,7 @@ defOperator(Module m, atom_t name, int type, int priority, int force)
   { PL_register_atom(name);
     addNewHTable(m->operators, (void *)name, op);
   }
-  UNLOCK();
+  PL_UNLOCK(L_OP);
 
   return TRUE;
 }
@@ -574,17 +571,36 @@ current_op(Module m, int inherit,
   fail;
 }
 
+static int
+get_op_module(term_t a3, term_t name, Module *m ARG_LD)
+{ atom_t mname = 0;
+  Word p;
+
+  if ( !(p=stripModuleName(valTermRef(a3), &mname PASS_LD)) )
+    return FALSE;
+  *valTermRef(name) = linkVal(p);
+
+  if ( mname && !(*m=isCurrentModule(mname)) )
+    *m = MODULE_user;
+
+  return TRUE;
+}
+
+
 static
-PRED_IMPL("current_op", 3, current_op, PL_FA_NONDETERMINISTIC|PL_FA_TRANSPARENT|PL_FA_ISO)
+PRED_IMPL("current_op", 3, current_op,
+	  PL_FA_NONDETERMINISTIC|PL_FA_TRANSPARENT|PL_FA_ISO)
 { PRED_LD
   Module m = MODULE_parse;
+  term_t name = A3;
 
   if ( CTX_CNTRL != FRG_CUTTED )
-  { if ( !PL_strip_module(A3, &m, A3) )
+  { if ( !(name = PL_new_term_ref()) ||
+	 !get_op_module(A3, name, &m PASS_LD) )
       return FALSE;
   }
 
-  return current_op(m, TRUE, A1, A2, A3, PL__ctx PASS_LD);
+  return current_op(m, TRUE, A1, A2, name, PL__ctx PASS_LD);
 }
 
 /** '$local_op'(?Precedence, ?Type, ?Name) is nondet.
@@ -599,13 +615,15 @@ static
 PRED_IMPL("$local_op", 3, local_op, PL_FA_NONDETERMINISTIC|PL_FA_TRANSPARENT)
 { PRED_LD
   Module m = MODULE_user;
+  term_t name = A3;
 
   if ( CTX_CNTRL != FRG_CUTTED )
-  { if ( !PL_strip_module(A3, &m, A3) )
+  { if ( !(name = PL_new_term_ref()) ||
+	 !get_op_module(A3, name, &m PASS_LD) )
       return FALSE;
   }
 
-  return current_op(m, FALSE, A1, A2, A3, PL__ctx PASS_LD);
+  return current_op(m, FALSE, A1, A2, name, PL__ctx PASS_LD);
 }
 
 
