@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2015, University of Amsterdam
+    Copyright (c)  1985-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -37,6 +37,7 @@
 #define O_DEBUG 1			/* include checkData() */
 #endif
 #include "pl-incl.h"
+#include "os/pl-cstack.h"
 
 
 		/********************************
@@ -79,18 +80,21 @@ resetProlog(int clear_stacks)
 }
 
 
-static void
+static int
 restore_after_exception(term_t except)
 { GET_LD
   atom_t a;
+  int rc = TRUE;
 
   tracemode(FALSE, NULL);
   debugmode(DBG_OFF, NULL);
   setPrologFlagMask(PLFLAG_LASTCALL);
   if ( PL_get_atom(except, &a) && a == ATOM_aborted )
-  { callEventHook(PLEV_ABORT);
-    printMessage(ATOM_informational, PL_ATOM, ATOM_aborted);
+  { rc = ( callEventHook(PLEV_ABORT) &&
+	   printMessage(ATOM_informational, PL_ATOM, ATOM_aborted) );
   }
+
+  return rc;
 }
 
 
@@ -156,7 +160,7 @@ the debugger.  Restores I/O and debugger on exit.  The Prolog  predicate
 static int
 pl_break1(atom_t goal)
 { GET_LD
-  int rc;
+  int rc = TRUE;
   int old_level = LD->break_level;
 
   IOSTREAM *inSave  = Scurin;
@@ -174,19 +178,19 @@ pl_break1(atom_t goal)
 
   LD->break_level++;
   if ( LD->break_level > 0 )
-  { printMessage(ATOM_informational,
-		 PL_FUNCTOR, FUNCTOR_break2,
-	           PL_ATOM, ATOM_begin,
-		   PL_INT,  LD->break_level);
+  { rc = printMessage(ATOM_informational,
+		      PL_FUNCTOR, FUNCTOR_break2,
+		        PL_ATOM, ATOM_begin,
+		        PL_INT,  LD->break_level);
   }
 
-  rc = query_loop(goal, TRUE);
+  rc = rc && (query_loop(goal, TRUE) == TRUE);
 
   if ( LD->break_level > 0 )
-  { printMessage(ATOM_informational,
-		 PL_FUNCTOR, FUNCTOR_break2,
-	           PL_ATOM, ATOM_end,
-		   PL_INT,  LD->break_level);
+  { rc = rc && printMessage(ATOM_informational,
+			    PL_FUNCTOR, FUNCTOR_break2,
+			      PL_ATOM, ATOM_end,
+			      PL_INT,  LD->break_level);
   }
   LD->break_level = old_level;
 
@@ -198,7 +202,7 @@ pl_break1(atom_t goal)
   Scurout = outSave;
   Scurin  = inSave;
 
-  return rc == TRUE;
+  return rc;
 }
 
 
@@ -405,6 +409,10 @@ abortProlog(void)
     PL_close_foreign_frame(fid);
   }
 
+#ifdef O_MAINTENANCE
+  save_backtrace("abort");
+#endif
+
   return rc;
 }
 
@@ -580,7 +588,7 @@ last_arg:
     if ( p2 == p )
       printk(context, "Reference to same address");
     if ( !onLocal(p2) && !onGlobal(p2) )
-      printk(context, "Illegal reference pointer at 0x%x --> 0x%x", p, p2);
+      printk(context, "Illegal reference pointer at %p --> %p", p, p2);
 
     p = p2;
   }
@@ -601,13 +609,13 @@ last_arg:
 
 					/* See argument_stack_to_term_refs() */
     if ( !onGlobal(p) && (!gc_status.active || p < (Word)environment_frame) )
-      printk(context, "attvar: not on global stack: 0x%x", p);
+      printk(context, "attvar: not on global stack: %p", p);
     if ( !onGlobal(p2) )
-      printk(context, "attvar: attribute not on global stack: 0x%x --> 0x%x", p, p2);
+      printk(context, "attvar: attribute not on global stack: %p --> %p", p, p2);
     if ( p == p2 )
-      printk(context, "attvar: self-reference: 0x%x", p);
+      printk(context, "attvar: self-reference: %p", p);
     if ( !(context->flags&CHK_DATA_NOATTVAR_CHAIN) && !on_attvar_chain(p) )
-      printk(context, "attvar: not on attvar chain: 0x%x", p);
+      printk(context, "attvar: not on attvar chain: %p", p);
 
     p = p2;
     goto last_arg;
@@ -641,11 +649,11 @@ last_arg:
 
 	if ( sz != (len=strlen(s)) )
 	{ if ( sz < len )
-	    printk(context, "String has inconsistent length: 0x%x", *p);
+	    printk(context, "String has inconsistent length: %p", *p);
 	  else if ( s[sz] )
-	    printk(context, "String not followed by NUL-char: 0x%x", *p);
+	    printk(context, "String not followed by NUL-char: %p", *p);
 /*	else
-	    printf("String contains NUL-chars: 0x%x", *p);
+	    printf("String contains NUL-chars: %p", *p);
 */
 	}
       } else
@@ -656,9 +664,9 @@ last_arg:
 
 	if ( sz != (len=wcslen(s)) )
 	{ if ( sz < len )
-	    printk(context, "String has inconsistent length: 0x%x", *p);
+	    printk(context, "String has inconsistent length: %p", *p);
 	  else if ( s[sz] )
-	    printk(context, "String not followed by NUL-char: 0x%x", *p);
+	    printk(context, "String not followed by NUL-char: %p", *p);
 	}
       }
       return key + *addressIndirect(*p);
@@ -690,7 +698,7 @@ last_arg:
 					/* now it should be a term */
   if ( tag(*p) != TAG_COMPOUND ||
        storage(*p) != STG_GLOBAL )
-    printk(context, "Illegal term at: %p: 0x%x", p, *p);
+    printk(context, "Illegal term at: %p: %p", p, *p);
 
   if ( is_marked(p) )
   { context->recursive++;
@@ -704,11 +712,11 @@ last_arg:
 
     if ( tag(f->definition) != TAG_ATOM ||
          storage(f->definition) != STG_GLOBAL )
-      printk(context, "Illegal functor: 0x%x", *p);
+      printk(context, "Illegal functor: %p", *p);
     if ( f->definition & MARK_MASK )
-      printk(context, "functor with mark: 0x%x", *p);
+      printk(context, "functor with mark: %p", *p);
     if ( f->definition & FIRST_MASK )
-      printk(context, "functor with first: 0x%x", *p);
+      printk(context, "functor with first: %p", *p);
     arity = arityFunctor(f->definition);
     if ( arity < 0 )
       printk(context, "Illegal arity (%d)", arity);
