@@ -1088,7 +1088,7 @@ lingerClauseRef(ClauseRef cref)
 
 static int activePredicate(const Definition *defs, const Definition def);
 
-void
+static void
 gcClauseRefs(void)
 { ClauseRef cref;
 
@@ -1397,15 +1397,38 @@ void
 unallocClause(Clause c)
 { ATOMIC_SUB(&GD->statistics.codes, c->code_size);
   ATOMIC_DEC(&GD->statistics.clauses);
+
+#ifdef ALLOC_DEBUG
+#define ALLOC_FREE_MAGIC 0xFB
+  size_t size = sizeofClause(c->code_size);
+  memset(c, ALLOC_FREE_MAGIC, size);
+#endif
+
   PL_free(c);
 }
 
 
+#ifdef O_DEBUG_ATOMGC
+void
+unregister_atom_clause(atom_t a)
+{ PL_unregister_atom(a);
+}
+
+void
+register_atom_clause(atom_t a)
+{ PL_register_atom(a);
+}
+#endif
+
 void
 freeClause(Clause c)
 {
 #ifdef O_ATOMGC
+#ifdef O_DEBUG_ATOMGC
+  forAtomsInClause(c, unregister_atom_clause);
+#else
   forAtomsInClause(c, PL_unregister_atom);
+#endif
 #endif
 
   if ( true(c, DBREF_CLAUSE) )		/* will be freed from symbol */
@@ -1434,6 +1457,21 @@ announceErasedClause(Clause clause)
 }
 
 
+static void
+freeLingeringDefinition(Definition def, DirtyDefInfo ddi)
+{ linger_list *c;
+
+  do
+  { if ( !(c=def->lingering) )
+      return;
+    if ( ddi->oldest_generation != GEN_MAX )
+      return;
+  } while( !COMPARE_AND_SWAP(&def->lingering, c, NULL) );
+
+  free_lingering(c);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 cleanDefinition()
     This function has two tasks. If the predicate needs to be rehashed,
@@ -1448,6 +1486,11 @@ cleanDefinition() keep track of this, but  in the meanwhile the previous
 may change due to an assert. Now that we are in the locked region we can
 search for the real previous, using   the  one from cleanDefinition() as
 the likely candidate.
+
+The `ddi->oldest_generation` contains the latest  marked generation that
+was found or GEN_MAX  if  the  predicate   is  not  active.  The `start`
+generation contains the generation when pl_garbage_collect_clauses() was
+started.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int	mustCleanDefinition(const Definition def);
@@ -1471,9 +1514,10 @@ find_prev(Definition def, ClauseRef prev, ClauseRef cref)
 
 
 static size_t
-cleanDefinition(Definition def, gen_t marked, gen_t start, int *rcp)
+cleanDefinition(Definition def, DirtyDefInfo ddi, gen_t start, int *rcp)
 { GET_LD
   size_t removed = 0;
+  gen_t marked = ddi->oldest_generation;
   gen_t active = start < marked ? start : marked;
 
   DEBUG(CHK_SECURE, checkDefinition(def));
@@ -1520,8 +1564,7 @@ cleanDefinition(Definition def, gen_t marked, gen_t start, int *rcp)
       cleanClauseIndexes(def, active);
       UNLOCKDEF(def);
     }
-    if ( marked == GEN_MAX && def->lingering )
-      free_lingering(&def->lingering);
+    freeLingeringDefinition(def, ddi);
     release_def(def);
 
     DEBUG(CHK_SECURE, checkDefinition(def));
@@ -2090,9 +2133,7 @@ pl_garbage_collect_clauses(void)
 
 		if ( false(def, P_FOREIGN) &&
 		     def->impl.clauses.erased_clauses > 0 )
-		{ size_t del = cleanDefinition(def,
-					       ddi->oldest_generation,
-					       start_gen, &rc);
+		{ size_t del = cleanDefinition(def, ddi, start_gen, &rc);
 
 		  removed += del;
 		  DEBUG(MSG_CGC_PRED,
@@ -2495,15 +2536,12 @@ PRED_IMPL("retract", 1, retract,
 	fail;				/* no clauses */
       }
 
-      startCritical;
       enterDefinition(def);			/* reference the predicate */
-      pushPredicateAccess(def, generationFrame(environment_frame));
+      setGenerationFrameVal(environment_frame, pushPredicateAccess(def));
       cref = firstClause(argv, environment_frame, def, &ctxbuf.chp PASS_LD);
       if ( !cref )
       { popPredicateAccess(def);
 	leaveDefinition(def);
-	if ( !endCritical )
-	  fail;
 	fail;
       }
 
@@ -2513,29 +2551,18 @@ PRED_IMPL("retract", 1, retract,
     } else
     { ctx  = CTX_PTR;
       cref = nextClause(&ctx->chp, argv, environment_frame, ctx->def);
-      startCritical;
     }
 
     if ( !(fid = PL_open_foreign_frame()) )
     { free_retract_context(ctx PASS_LD);
-      endCritical;
       return FALSE;
     }
 
-    /* ctx->cref is the first candidate; next is the next one */
-
     while( cref )
     { if ( decompile(cref->value.clause, cl, 0) )
       { if ( retractClauseDefinition(ctx->def, cref->value.clause) ||
 	     CTX_CNTRL != FRG_FIRST_CALL )
-	{ if ( !endCritical )
-	  { free_retract_context(ctx PASS_LD);
-	    PL_close_foreign_frame(fid);
-
-	    return FALSE;
-	  }
-
-	  if ( !ctx->chp.cref )		/* deterministic last one */
+	{ if ( !ctx->chp.cref )		/* deterministic last one */
 	  { free_retract_context(ctx PASS_LD);
 	    PL_close_foreign_frame(fid);
 	    return TRUE;
@@ -2547,18 +2574,19 @@ PRED_IMPL("retract", 1, retract,
 	  PL_close_foreign_frame(fid);
 	  ForeignRedoPtr(ctx);
 	} else
-	{ setGenerationFrame(environment_frame, global_generation());
+	{ setGenerationFrame(environment_frame);
 	}
       }
 
-      PL_rewind_foreign_frame(fid);
+      if ( PL_exception(0) )
+	break;
 
+      PL_rewind_foreign_frame(fid);
       cref = nextClause(&ctx->chp, argv, environment_frame, ctx->def);
     }
 
     PL_close_foreign_frame(fid);
     free_retract_context(ctx PASS_LD);
-    endCritical;
     return FALSE;
   }
 }
@@ -2628,7 +2656,7 @@ pl_retractall(term_t head)
 
   startCritical;
   enterDefinition(def);
-  pushPredicateAccess(def, generationFrame(environment_frame));
+  setGenerationFrameVal(environment_frame, pushPredicateAccess(def));
   fid = PL_open_foreign_frame();
 
   DEBUG(CHK_SECURE, checkDefinition(def));
@@ -3306,7 +3334,11 @@ PRED_IMPL("copy_predicate_clauses", 2, copy_predicate_clauses, PL_FA_TRANSPARENT
       if ( def->module != copy_def->module )
 	remoduleClause(copy, def->module, copy_def->module);
 #ifdef O_ATOMGC
+#ifdef O_DEBUG_ATOMGC
+      forAtomsInClause(copy, register_atom_clause);
+#else
       forAtomsInClause(copy, PL_register_atom);
+#endif
 #endif
       assertProcedure(to, copy, CL_END PASS_LD);
     }
@@ -3360,9 +3392,10 @@ checkDefinition(Definition def)
   unsigned int nc, indexed = 0;
   ClauseRef cref;
   unsigned int erased = 0;
+  Definition old;
 
 						/* check basic clause list */
-  acquire_def(def);
+  acquire_def2(def, old);
   for(nc=0, cref = def->impl.clauses.first_clause; cref; cref=cref->next)
   { Clause clause = cref->value.clause;
 
@@ -3374,7 +3407,7 @@ checkDefinition(Definition def)
     { erased++;
     }
   }
-  release_def(def);
+  release_def2(def, old);
 
   assert(nc == def->impl.clauses.number_of_clauses);
   assert(erased == def->impl.clauses.erased_clauses);
