@@ -399,6 +399,32 @@ to the main thread.
 #undef LD
 #define LD LOCAL_LD
 
+static int
+is_fatal_signal(int sig)
+{ switch(sig)
+  {
+#ifdef SIGFPE
+    case SIGFPE:
+#endif
+#ifdef SIGSEGV
+    case SIGSEGV:
+#if defined(SIGBUS) && SIGBUS != SIGSEGV
+    case SIGBUS:
+#endif
+#endif
+#ifdef SIGILL
+    case SIGILL:
+#endif
+#ifdef SIGSYS
+    case SIGSYS:
+#endif
+      return TRUE;
+  }
+
+  return FALSE;
+}
+
+
 void
 dispatch_signal(int sig, int sync)
 { GET_LD
@@ -417,10 +443,16 @@ dispatch_signal(int sig, int sync)
   }
 
   DEBUG(MSG_SIGNAL,
-	Sdprintf("Got signal %d in thread %d (=%d) %s\n",
-		 sig, LD->thread.info->pl_tid,
-		 pthread_self(),
-		 sync ? " (sync)" : " (async)"));
+	{ const pl_wchar_t *name = L"";
+	  int tid = LD->thread.info->pl_tid;
+	  atom_t alias;
+
+	  if ( PL_get_thread_alias(tid, &alias) )
+	    name = PL_atom_wchars(alias, NULL);
+	  Sdprintf("Got signal %d in thread %d (%Ws) %s\n",
+		   sig, tid, name,
+		   sync ? " (sync)" : " (async)");
+	});
 #else
   DEBUG(MSG_SIGNAL,
 	Sdprintf("Got signal %d %s\n",
@@ -436,28 +468,24 @@ dispatch_signal(int sig, int sync)
   saved_current_signal = LD->signal.current;
   saved_sync = LD->signal.is_sync;
 
-  switch(sig)
-  { case SIGFPE:
-    case SIGSEGV:
-#if defined(SIGBUS) && SIGBUS != SIGSEGV
-    case SIGBUS:
-#endif
-      if ( sig == LD->signal.current )
-	sysError("Recursively received fatal signal %d", sig);
-  }
+  if ( is_fatal_signal(sig) && sig == LD->signal.current )
+    sysError("Recursively received fatal signal %d", sig);
 
   if ( gc_status.active && sig < SIG_PROLOG_OFFSET )
   { fatalError("Received signal %d (%s) while in %ld-th garbage collection",
 	       sig, signal_name(sig), gc_status.collections);
   }
 
-  if ( LD->critical || (true(sh, PLSIG_SYNC) && !sync) )
+  if ( (LD->critical || (true(sh, PLSIG_SYNC) && !sync)) &&
+       !is_fatal_signal(sig) )
   { PL_raise(sig);			/* wait for better times! */
     return;
   }
 
   if ( !(fid = PL_open_signal_foreign_frame(sync)) )
-  { PL_raise(sig);			/* no space; wait */
+  { if ( is_fatal_signal(sig) )
+      sigCrashHandler(sig);		/* should not return */
+    PL_raise(sig);			/* no space; wait */
     return;
   }
 
@@ -533,6 +561,16 @@ dispatch_signal(int sig, int sync)
 
   if ( !sync )
     unblockGC(0 PASS_LD);
+
+					/* we cannot return.  First try */
+					/* longjmp.  If that fails, crash */
+  if ( is_fatal_signal(sig) )
+  { if ( exception_term )
+    { PL_rethrow();
+      sigCrashHandler(sig);
+    }
+    exit(4);
+  }
 }
 
 
@@ -1062,6 +1100,45 @@ handleSignals(ARG1_LD)
 }
 
 
+#ifdef SIG_ALERT
+static
+PRED_IMPL("prolog_alert_signal", 2, prolog_alert_signal, 0)
+{ PRED_LD
+  const char *sname = signal_name(GD->signals.sig_alert);
+  int rc;
+
+  if ( strcmp(sname, "unknown") == 0 )
+    rc = PL_unify_integer(A1, GD->signals.sig_alert);
+  else
+    rc = PL_unify_atom_chars(A1, sname);
+
+  if ( rc )
+  { if ( PL_compare(A1,A2) == CMP_EQUAL )
+    { return TRUE;
+    } else
+    { int new;
+
+      if ( (PL_get_integer(A2, &new) && new == 0) ||
+	   PL_get_signum_ex(A2, &new) )
+      { if ( GD->signals.sig_alert )
+	{ unprepareSignal(GD->signals.sig_alert);
+	  GD->signals.sig_alert = 0;
+	}
+	if ( new )
+	{ GD->signals.sig_alert = new;
+	  PL_signal(GD->signals.sig_alert|PL_SIGNOFRAME, alert_handler);
+	}
+
+	return TRUE;
+      }
+    }
+  }
+
+  return FALSE;
+}
+#endif
+
+
 int
 endCritical__LD(ARG1_LD)
 { if ( exception_term )
@@ -1757,7 +1834,10 @@ PRED_IMPL("$set_prolog_stack", 4, set_prolog_stack, 0)
 		 *******************************/
 
 BeginPredDefs(setup)
-  PRED_DEF("$set_prolog_stack", 4, set_prolog_stack, 0)
-  PRED_DEF("$on_signal", 4, on_signal, 0)
-  PRED_DEF("trim_stacks", 0, trim_stacks, 0)
+  PRED_DEF("$set_prolog_stack",	  4, set_prolog_stack,	  0)
+  PRED_DEF("$on_signal",	  4, on_signal,		  0)
+  PRED_DEF("trim_stacks",	  0, trim_stacks,	  0)
+#ifdef SIG_ALERT
+  PRED_DEF("prolog_alert_signal", 2, prolog_alert_signal, 0)
+#endif
 EndPredDefs
