@@ -47,6 +47,10 @@
 #define SIG_ALERT  SIGUSR2
 #endif
 
+#define GCREQUEST_AGC   0x01		/* GD->thread.gc.requests */
+#define GCREQUEST_CGC   0x02
+#define GCREQUEST_ABORT 0x04
+
 typedef enum
 { LDATA_IDLE = 0,
   LDATA_SIGNALLED,
@@ -54,18 +58,19 @@ typedef enum
   LDATA_ANSWERED
 } ldata_status_t;
 
-#define THREAD_STATUS_INVALID(s) ((s) == PL_THREAD_UNUSED || \
-				  (s) == PL_THREAD_RESERVED)
+#define THREAD_STATUS_INVALID(s) ((int)(s) < (int)PL_THREAD_RUNNING)
 
 typedef enum
 { PL_THREAD_UNUSED = 0,			/* no thread on this slot */
+  PL_THREAD_RESERVED,			/* allocated but not yet created */
+  PL_THREAD_JOINED,			/* just joined */
+					/* normal states */
   PL_THREAD_RUNNING,			/* a normally running one */
   PL_THREAD_EXITED,			/* died with thread_exit/1 */
   PL_THREAD_SUCCEEDED,			/* finished with Yes */
   PL_THREAD_FAILED,			/* finished with No */
   PL_THREAD_EXCEPTION,			/* finished with exception */
   PL_THREAD_NOMEM,			/* couldn't start due no-memory */
-  PL_THREAD_RESERVED,			/* allocated but not yet created */
   PL_THREAD_CREATED,			/* just created */
 } thread_status;
 
@@ -88,7 +93,7 @@ typedef struct _PL_thread_info_t
   size_t	    global_size;
   size_t	    trail_size;
   size_t	    stack_size;		/* system (C-) stack */
-  int		    (*cancel)(int id);	/* cancel function */
+  rc_cancel	    (*cancel)(int id);	/* cancel function */
   unsigned short    open_count;		/* for PL_thread_detach_engine() */
   unsigned	    detached      : 1;	/* detached thread */
   unsigned	    debug         : 1;	/* thread can be debugged */
@@ -213,9 +218,10 @@ extern counting_mutex _PL_mutexes[];	/* Prolog mutexes */
 #define L_SORTR        22
 #define L_UMUTEX       23
 #define L_INIT_ATOMS   24
+#define L_CGCGEN       25
 #ifdef __WINDOWS__
-#define L_DDE	       25
-#define L_CSTACK       26
+#define L_DDE	       26
+#define L_CSTACK       27
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -443,7 +449,8 @@ COMMON(Atom**)	pl_atom_buckets_in_use(void);
 COMMON(Definition*)	predicates_in_use(void);
 COMMON(int)	pl_functor_table_in_use(FunctorTable functor_table);
 COMMON(int)	pl_kvs_in_use(KVS kvs);
-COMMON(int)	pushPredicateAccess__LD(Definition def, gen_t gen ARG_LD);
+COMMON(void)	cgcActivatePredicate__LD(Definition def, gen_t gen ARG_LD);
+COMMON(gen_t)	pushPredicateAccess__LD(Definition def ARG_LD);
 COMMON(void)	popPredicateAccess__LD(Definition def ARG_LD);
 COMMON(size_t)	popNPredicateAccess__LD(size_t n ARG_LD);
 COMMON(void)	markAccessedPredicates(PL_local_data_t *ld);
