@@ -931,11 +931,11 @@ writeFrameGoal(IOSTREAM *out, LocalFrame frame, Code PC, unsigned int flags)
     if ( rc )
     { IOSTREAM *old = Suser_error;
       Suser_error = out;
-      printMessage(ATOM_debug,
-		   PL_FUNCTOR, FUNCTOR_frame3,
-		     PL_TERM, fr,
-		     PL_TERM, port,
-		     PL_TERM, pc);
+      rc = printMessage(ATOM_debug,
+			PL_FUNCTOR, FUNCTOR_frame3,
+			  PL_TERM, fr,
+			  PL_TERM, port,
+			  PL_TERM, pc);
       Suser_error = old;
     }
   } else
@@ -1294,14 +1294,20 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
 	  PL_warning("prolog_trace_interception/4: bad argument to retry/1");
       }
     } else if ( (ex=PL_exception(qid)) )
-    { atom_t a;
-
-      if ( PL_get_atom(ex, &a) && a == ATOM_aborted )
+    { if ( classify_exception(ex) == EXCEPT_ABORT )
       { rval = ACTION_ABORT;
       } else
-      { printMessage(ATOM_error, PL_TERM, ex);
-	nodebug = TRUE;
-	rval = ACTION_CONTINUE;
+      { if ( printMessage(ATOM_error, PL_TERM, ex) )
+	{ nodebug = TRUE;
+	  rval = ACTION_CONTINUE;
+	} else if ( classify_exception(exception_term) >= EXCEPT_TIMEOUT )
+	{ PL_clear_exception();
+	  rval = ACTION_ABORT;
+	} else
+	{ PL_clear_exception();
+	  nodebug = TRUE;
+	  rval = ACTION_CONTINUE;
+	}
       }
     }
 
@@ -1656,14 +1662,16 @@ interruptHandler(int sig)
   Sreset();
 again:
   if ( safe )
-  { printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1, PL_ATOM, ATOM_begin);
+  { if ( !printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1,
+		                     PL_ATOM, ATOM_begin) )
+      PL_clear_exception();
   } else
   { if ( first )
     { first = FALSE;
       Sfprintf(Sdout,
 	       "\n"
 	       "WARNING: By typing Control-C twice, you have forced an asynchronous\n"
-	       "WARNING: interrupt.  Your only SAFE operation are: c(ontinue), p(id),\n"
+	       "WARNING: interrupt.  Your only SAFE operations are: c(ontinue), p(id),\n"
 	       "WARNING: s(stack) and e(xit).  Notably a(abort) often works, but\n"
 	       "WARNING: leaves the system in an UNSTABLE state\n\n");
     }
@@ -1688,7 +1696,10 @@ again:
 		}
 		goto again;
     case 'c':	if ( safe )
-		{ printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1, PL_ATOM, ATOM_end);
+		{ if ( !printMessage(ATOM_debug,
+				     PL_FUNCTOR, FUNCTOR_interrupt1,
+				       PL_ATOM, ATOM_end) )
+		    PL_clear_exception();
 		} else
 		{ Sfprintf(Sdout, "continue\n");
 		}
@@ -1714,9 +1725,10 @@ again:
 #ifdef O_DEBUGGER
     case 't':	if ( safe )
 		{ Sfprintf(Sdout, "trace\n");
-		  printMessage(ATOM_debug,
-			       PL_FUNCTOR, FUNCTOR_interrupt1,
-			         PL_ATOM, ATOM_trace);
+		  if ( !printMessage(ATOM_debug,
+				     PL_FUNCTOR, FUNCTOR_interrupt1,
+				       PL_ATOM, ATOM_trace) )
+		    PL_clear_exception();
 		  pl_trace();
 		  break;
 		} else
@@ -1784,9 +1796,9 @@ tracemode(int doit, int *old)
 
   if ( debugstatus.tracing != doit )
   { debugstatus.tracing = doit;
-    printMessage(ATOM_silent,
-		 PL_FUNCTOR_CHARS, "trace_mode", 1,
-		   PL_ATOM, doit ? ATOM_on : ATOM_off);
+    return printMessage(ATOM_silent,
+			PL_FUNCTOR_CHARS, "trace_mode", 1,
+			  PL_ATOM, doit ? ATOM_on : ATOM_off);
   }
   if ( doit )				/* make sure trace works inside skip */
   { debugstatus.skiplevel = SKIP_VERY_DEEP;
@@ -1881,9 +1893,9 @@ debugmode(debug_type doit, debug_type *old)
     }
     debugstatus.debugging = doit;
     updateAlerted(LD);
-    printMessage(ATOM_silent,
-		 PL_FUNCTOR_CHARS, "debug_mode", 1,
-		   PL_ATOM, doit ? ATOM_on : ATOM_off);
+    return printMessage(ATOM_silent,
+			PL_FUNCTOR_CHARS, "debug_mode", 1,
+			  PL_ATOM, doit ? ATOM_on : ATOM_off);
   }
 
   return TRUE;
@@ -1985,15 +1997,15 @@ pl_spy(term_t p)
     { LOCKDEF(def);
       set(def, SPY_ME);
       UNLOCKDEF(def);
-      printMessage(ATOM_informational,
-		   PL_FUNCTOR_CHARS, "spy", 1,
-		     PL_TERM, p);
+      if ( !printMessage(ATOM_informational,
+			 PL_FUNCTOR_CHARS, "spy", 1,
+			   PL_TERM, p) )
+	return FALSE;
     }
-    debugmode(DBG_ALL, NULL);
-    succeed;
+    return debugmode(DBG_ALL, NULL);
   }
 
-  fail;
+  return FALSE;
 }
 
 word
@@ -2008,14 +2020,14 @@ pl_nospy(term_t p)
     { LOCKDEF(def);
       clear(def, SPY_ME);
       UNLOCKDEF(def);
-      printMessage(ATOM_informational,
-		   PL_FUNCTOR_CHARS, "nospy", 1,
-		     PL_TERM, p);
+      return printMessage(ATOM_informational,
+			  PL_FUNCTOR_CHARS, "nospy", 1,
+			    PL_TERM, p);
     }
-    succeed;
+    return TRUE;
   }
 
-  fail;
+  return FALSE;
 }
 
 word
@@ -2204,9 +2216,12 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
 
 	deRef2(argv+n, a);
 	if ( isVar(*a) && onStack(local, a) )
-	  Trail(a, makeRef(argp));
-	else
+	{ LTrail(a);
+	  *a = makeRef(argp);
+	} else
+	{ GTrail(argp);
 	  *argp = (needsRef(*a) ? makeRef(a) : *a);
+	}
       }
     }
   } else if ( key == ATOM_predicate_indicator )
@@ -2452,8 +2467,13 @@ delayEvents(void)
 }
 
 
+/* Returns
+     -1: an exception occurred while sending events
+      N: number of events sent
+*/
+
 int
-sendDelayedEvents(void)
+sendDelayedEvents(int noerror)
 { GET_LD
   int sent = 0;
 
@@ -2465,18 +2485,21 @@ sendDelayedEvents(void)
     LD->event.buffered = NULL;
 
     for(; count-- > 0; dev++)
-    { switch(dev->type)
-      { case PLEV_BREAK:
-	case PLEV_NOBREAK:
-	  callEventHook(dev->type, dev->value.pc.clause, dev->value.pc.offset);
-	  sent++;
-	  break;
-	case PLEV_ERASED_CLAUSE:
-	  callEventHook(dev->type, dev->value.clause.clause);
-	  sent++;
-	  break;
-	default:
-	  assert(0);
+    { if ( noerror )
+      { switch(dev->type)
+	{ case PLEV_BREAK:
+	  case PLEV_NOBREAK:
+	    noerror = callEventHook(dev->type,
+				    dev->value.pc.clause, dev->value.pc.offset);
+	    sent++;
+	    break;
+	  case PLEV_ERASED_CLAUSE:
+	    noerror = callEventHook(dev->type, dev->value.clause.clause);
+	    sent++;
+	    break;
+	  default:
+	    assert(0);
+	}
       }
     }
 
@@ -2484,7 +2507,7 @@ sendDelayedEvents(void)
     free(b);
   }
 
-  return sent;
+  return noerror ? sent	: -1;
 }
 
 
@@ -2492,119 +2515,11 @@ int
 PL_call_event_hook(pl_event_type ev, ...)
 { if ( PROCEDURE_event_hook1->definition->impl.any &&
        GD->cleaning != CLN_DATA )
-  { GET_LD
-    wakeup_state wstate;
+  { va_list args;
     int rc;
-    va_list args;
-    term_t arg;
-
-    if ( LD->event.delay_nesting )
-    { va_start(args, ev);
-      delayEvent(ev, args);
-      va_end(args);
-      return TRUE;
-    }
-
-    if ( !saveWakeup(&wstate, TRUE PASS_LD) )
-      return FALSE;
-    arg = PL_new_term_ref();
 
     va_start(args, ev);
-    switch(ev)
-    { case PLEV_ABORT:
-      { rc = PL_unify_atom(arg, ATOM_abort);
-	break;
-      }
-      case PLEV_ERASED_CLAUSE:
-      {	Clause cl = va_arg(args, Clause);	/* object erased */
-	term_t dbref = PL_new_term_ref();
-
-	rc = (  PL_unify_clref(dbref, cl) &&
-		PL_unify_term(arg,
-			      PL_FUNCTOR, FUNCTOR_erased1,
-			        PL_TERM, dbref)
-	     );
-	break;
-      }
-      case PLEV_ERASED_RECORD:
-      {	RecordRef r = va_arg(args, RecordRef);	/* object erased */
-	term_t dbref = PL_new_term_ref();
-
-	rc = (  PL_unify_recref(dbref, r) &&
-		PL_unify_term(arg,
-			      PL_FUNCTOR, FUNCTOR_erased1,
-			        PL_TERM, dbref)
-	     );
-	break;
-      }
-      case PLEV_DEBUGGING:
-      { int dbg = va_arg(args, int);
-
-	rc = PL_unify_term(arg,
-			   PL_FUNCTOR, FUNCTOR_debugging1,
-			     PL_ATOM, dbg ? ATOM_true : ATOM_false);
-	break;
-      }
-      case PLEV_TRACING:
-      { int trc = va_arg(args, int);
-
-	rc = PL_unify_term(arg,
-			   PL_FUNCTOR, FUNCTOR_tracing1,
-			     PL_ATOM, trc ? ATOM_true : ATOM_false);
-	break;
-      }
-      case PLEV_BREAK:
-      case PLEV_NOBREAK:
-      { Clause clause = va_arg(args, Clause);
-	int offset = va_arg(args, int);
-	term_t cref = PL_new_term_ref();
-
-
-	rc = ( PL_unify_clref(cref, clause) &&
-	       PL_unify_term(arg,
-			     PL_FUNCTOR, FUNCTOR_break3,
-			       PL_TERM, cref,
-			       PL_INT, offset,
-			       PL_ATOM, ev == PLEV_BREAK ? ATOM_true
-						         : ATOM_false)
-	     );
-	break;
-      }
-      case PLEV_FRAMEFINISHED:
-      { LocalFrame fr = va_arg(args, LocalFrame);
-	term_t ref = PL_new_term_ref();
-
-	PL_put_frame(ref, fr);
-	rc = PL_unify_term(arg,
-			   PL_FUNCTOR, FUNCTOR_frame_finished1,
-			     PL_TERM, ref);
-	break;
-      }
-#ifdef O_PLMT
-      case PL_EV_THREADFINISHED:
-      { PL_thread_info_t *info = va_arg(args, PL_thread_info_t*);
-	term_t id;
-
-	rc = ( (id = PL_new_term_ref()) &&
-	       unify_thread_id(id, info) &&
-	       PL_unify_term(arg,
-			     PL_FUNCTOR_CHARS, "thread_finished", 1,
-			       PL_TERM, id)
-	     );
-
-	break;
-      }
-#endif
-      default:
-	rc = warning("callEventHook(): unknown event: %d", ev);
-        goto out;
-    }
-
-    if ( rc )
-      rc = PL_call_predicate(MODULE_user, FALSE, PROCEDURE_event_hook1, arg);
-
-  out:
-    restoreWakeup(&wstate PASS_LD);
+    rc = PL_call_event_hook_va(ev, args);
     va_end(args);
 
     return rc;
@@ -2613,6 +2528,132 @@ PL_call_event_hook(pl_event_type ev, ...)
   return TRUE;
 }
 
+
+/* Returns FALSE iff there is an exception inside the execution of the
+ * hook
+ */
+
+int
+PL_call_event_hook_va(pl_event_type ev, va_list args)
+{ GET_LD
+  wakeup_state wstate;
+  int rc;
+  term_t arg;
+
+  if ( LD->event.delay_nesting )
+  { delayEvent(ev, args);
+    return TRUE;
+  }
+
+  if ( !saveWakeup(&wstate, TRUE PASS_LD) )
+    return FALSE;
+  arg = PL_new_term_ref();
+
+  switch(ev)
+  { case PLEV_ABORT:
+    { rc = PL_unify_atom(arg, ATOM_abort);
+      break;
+    }
+    case PLEV_ERASED_CLAUSE:
+    { Clause cl = va_arg(args, Clause);		/* object erased */
+      term_t dbref = PL_new_term_ref();
+
+      rc = (  PL_unify_clref(dbref, cl) &&
+	      PL_unify_term(arg,
+			    PL_FUNCTOR, FUNCTOR_erased1,
+			      PL_TERM, dbref)
+	   );
+      break;
+    }
+    case PLEV_ERASED_RECORD:
+    { RecordRef r = va_arg(args, RecordRef);	/* object erased */
+      term_t dbref = PL_new_term_ref();
+
+      rc = (  PL_unify_recref(dbref, r) &&
+	      PL_unify_term(arg,
+			    PL_FUNCTOR, FUNCTOR_erased1,
+			      PL_TERM, dbref)
+	   );
+      break;
+    }
+    case PLEV_DEBUGGING:
+    { int dbg = va_arg(args, int);
+
+      rc = PL_unify_term(arg,
+			 PL_FUNCTOR, FUNCTOR_debugging1,
+			   PL_ATOM, dbg ? ATOM_true : ATOM_false);
+      break;
+    }
+    case PLEV_TRACING:
+    { int trc = va_arg(args, int);
+
+      rc = PL_unify_term(arg,
+			 PL_FUNCTOR, FUNCTOR_tracing1,
+			   PL_ATOM, trc ? ATOM_true : ATOM_false);
+      break;
+    }
+    case PLEV_BREAK:
+    case PLEV_NOBREAK:
+    { Clause clause = va_arg(args, Clause);
+      int offset = va_arg(args, int);
+      term_t cref = PL_new_term_ref();
+
+
+      rc = ( PL_unify_clref(cref, clause) &&
+	     PL_unify_term(arg,
+			   PL_FUNCTOR, FUNCTOR_break3,
+			     PL_TERM, cref,
+			     PL_INT, offset,
+			     PL_ATOM, ev == PLEV_BREAK ? ATOM_true
+						       : ATOM_false)
+	   );
+      break;
+    }
+    case PLEV_FRAMEFINISHED:
+    { LocalFrame fr = va_arg(args, LocalFrame);
+      term_t ref = PL_new_term_ref();
+
+      PL_put_frame(ref, fr);
+      rc = PL_unify_term(arg,
+			 PL_FUNCTOR, FUNCTOR_frame_finished1,
+			   PL_TERM, ref);
+      break;
+    }
+#ifdef O_PLMT
+    case PL_EV_THREADFINISHED:
+    { PL_thread_info_t *info = va_arg(args, PL_thread_info_t*);
+      term_t id;
+
+      rc = ( (id = PL_new_term_ref()) &&
+	     unify_thread_id(id, info) &&
+	     PL_unify_term(arg,
+			   PL_FUNCTOR_CHARS, "thread_finished", 1,
+			     PL_TERM, id)
+	   );
+
+      break;
+    }
+#endif
+    default:
+      rc = warning("callEventHook(): unknown event: %d", ev);
+      goto out;
+  }
+
+  if ( rc )
+  { rc = PL_call_predicate(MODULE_user, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
+			   PROCEDURE_event_hook1, arg);
+    if ( !rc && PL_exception(0) )
+      set(&wstate, WAKEUP_KEEP_URGENT_EXCEPTION);
+    else
+      rc = TRUE;				/* only FALSE on error */
+  }
+
+out:
+  restoreWakeup(&wstate PASS_LD);
+
+  return rc;
+}
+
 #endif /*O_DEBUGGER*/
 
 		 /*******************************
