@@ -1731,7 +1731,7 @@ gc(agc-2) :-		% not if concurrent: this is too simple.  There
 	    statistics(agc, AGC0),
 	    statistics(agc_gained, Gained0),
 	    forall(between(0, UpTo, X), atom_concat(foobar, X, _)),
-	    (	between(1, 6, X),
+	    (	between(1, 8, X),
 		(   statistics(agc, AGC1),
 		    AGC is AGC1 - AGC0,
 		    AGC > 5,
@@ -1744,6 +1744,11 @@ gc(agc-2) :-		% not if concurrent: this is too simple.  There
 		    fail
 		)
 	    ->	true
+	    ;	statistics(agc_gained, Gained1),
+		Gained is Gained1 - Gained0,
+		format(user_error,
+		       '~NWarning: gc(agc-2): reclaimed only ~D atoms~n',
+		       [Gained])
 	    )
 	;   true			% no atom-gc
 	).
@@ -2141,28 +2146,33 @@ os(setenv-1) :-
 		 *******************************/
 
 io(tell-1) :-
-	tell(test_x),
+	current_prolog_flag(pid, P),
+	atom_concat(test_x_, P, TestX),
+	atom_concat(test_y_, P, TestY),
+	tell(TestX),
 	format('~q.~n', [a]),
-	tell(test_y),
+	tell(TestY),
 	format('~q.~n', [b]),
-	tell(test_x),
+	tell(TestX),
 	format('~q.~n', [c]),
 	told,
-	tell(test_y),
+	tell(TestY),
 	told,
-	read_file_to_terms(test_x, [a,c], []),
-	read_file_to_terms(test_y, [b], []),
-	delete_file(test_x),
-	delete_file(test_y),
-	\+ stream_property(_, file_name(test_x)),
-	\+ stream_property(_, file_name(test_y)).
+	read_file_to_terms(TestX, [a,c], []),
+	read_file_to_terms(TestY, [b], []),
+	delete_file(TestX),
+	delete_file(TestY),
+	\+ stream_property(_, file_name(TestX)),
+	\+ stream_property(_, file_name(TestY)).
 
 io(tell-2) :-
+	current_prolog_flag(pid, P),
+	atom_concat(test_y_, P, TestY),
 	current_output(OrgOut),
 	open_null_stream(Out),
 	set_output(Out),
 	write(Out, x),
-	telling(Old), tell(test_y),
+	telling(Old), tell(TestY),
 	format('~q.~n', [b]),
 	told, tell(Old),
 	write(Out, y),
@@ -2170,8 +2180,8 @@ io(tell-2) :-
 	character_count(Out, 2),
 	close(Out),
 	set_output(OrgOut),
-	read_file_to_terms(test_y, [b], []),
-	delete_file(test_y).
+	read_file_to_terms(TestY, [b], []),
+	delete_file(TestY).
 
 
 		 /*******************************
@@ -2191,7 +2201,8 @@ popen(pwd-1) :-
 popen(cat-1) :-
 	(   current_prolog_flag(windows, true)
 	->  true		% there is *no* cmd.exe command like cat!?
-	;   File = 'pltest.txt',
+	;   current_prolog_flag(pid, Pid),
+	    format(atom(File), 'pltest-~w.txt', [Pid]),
 	    Text = 'Hello World',
 	    Cmd = cat,
 	    atomic_list_concat([Cmd, ' > ', File], Command),
@@ -2340,8 +2351,9 @@ file(cwd-1) :-
 	exists_directory(CWD),
 	same_file(CWD, '.').
 file(absfile-2) :-			% canonicaliseDir() caching issues
-	X = 'pl-test-x',
-	Y = 'pl-test-y',
+	current_prolog_flag(pid, Pid),
+	atom_concat('pl-test-x-', Pid, X),
+	atom_concat('pl-test-y-', Pid, Y),
 	atom_concat(X, '/file', XF),
 	atom_concat(Y, '/file', YF),
 	make_directory(X),
@@ -2371,8 +2383,13 @@ touch(File) :-
 		 *	UNICODE FILENAMES	*
 		 *******************************/
 
+unicode_file_name(Name) :-
+	current_prolog_flag(pid, Pid),
+	atom_codes(Name0, [1074, 1086, 1079, 1076, 1091, 1093, 1072]),
+	atomic_list_concat([Name0, -, Pid], Name).
+
 unicode_file(mkdir-1) :-			% create Cyrillic directory
-	atom_codes(Dir, [1074, 1086, 1079, 1076, 1091, 1093, 1072]),
+	unicode_file_name(Dir),
 	catch(delete_directory(Dir), _, true),
 	make_directory(Dir),
 	exists_directory(Dir),
@@ -2382,7 +2399,7 @@ unicode_file(mkdir-1) :-			% create Cyrillic directory
 	same_file(O2, Dir),
 	delete_directory(Dir).
 unicode_file(file-1) :-				% create Cyrillic file
-	atom_codes(File, [1074, 1086, 1079, 1076, 1091, 1093, 1072]),
+	unicode_file_name(File),
 	Term = hello(world),
 	catch(delete_file(File), _, true),
 	open(File, write, Out),
@@ -2395,7 +2412,7 @@ unicode_file(file-1) :-				% create Cyrillic file
 	Read =@= Term,
 	delete_file(File).
 unicode_file(absfile-1) :-
-	atom_codes(File, [1074, 1086, 1079, 1076, 1091, 1093, 1072]),
+	unicode_file_name(File),
 	absolute_file_name(File, Path),
 	file_directory_name(Path, Dir),
 	same_file(Dir, '.'),
@@ -2475,24 +2492,28 @@ wctype(code_type-6) :-
 		 *	      CONSULT		*
 		 *******************************/
 
-mk_include :-
-	open('test_included.pl', write, Out1),
+mk_include(Include, Included) :-
+	current_prolog_flag(pid, Pid),
+	format(atom(Included), 'test_included_~w.pl', [Pid]),
+	format(atom(Include), 'test_include_~w.pl', [Pid]),
+	file_name_extension(Base, pl, Included),
+	open(Included, write, Out1),
 	format(Out1, ':- dynamic foo/1.\n', []),
 	close(Out1),
 
-	open('test_include.pl', write, Out2),
-	format(Out2, ':- include(test_included).\n', []),
+	open(Include, write, Out2),
+	format(Out2, ':- include(~w).\n', [Base]),
 	format(Out2, 'foo(a).\n', []),
 	close(Out2).
 
 load_program(include-1) :-
-	mk_include,
+	mk_include(Include, Included),
 	abolish(foo, 1),
-	load_files(test_include, [silent(true)]),
+	load_files(Include, [silent(true)]),
 	assert(foo(b)),
 	findall(X, retract(foo(X)), [a,b]),
-	delete_file('test_included.pl'),
-	delete_file('test_include.pl').
+	delete_file(Included),
+	delete_file(Include).
 
 
 		 /*******************************
